### -*-awk-*-
### ====================================================================
###  @Awk-file{
###     author          = "Nelson H. F. Beebe",
###     version         = "0.07",
###     date            = "16 September 2013",
###     time            = "05:55:18 MDT",
###     filename        = "usmarc-to-bibtex.awk",
###     address         = "University of Utah
###                        Department of Mathematics, 110 LCB
###                        155 S 1400 E RM 233
###                        Salt Lake City, UT 84112-0090
###                        USA",
###     telephone       = "+1 801 581 5254",
###     FAX             = "+1 801 581 4148",
###     URL             = "http://www.math.utah.edu/~beebe",
###     checksum        = "41552 3525 13337 98707",
###     email           = "beebe@math.utah.edu, beebe@acm.org,
###                        beebe@computer.org (Internet)",
###     codetable       = "ISO/ASCII",
###     keywords        = "BibTeX; SUTRS library catalog records;
###                        USMARC library catalog records; Z39.50
###                        library catalog server",
###     license         = "public domain",
###     supported       = "yes",
###     docstring       = "Convert USMARC library catalog data to
###                        (rough) BibTeX form that can be further
###                        cleaned up by bibclean, biborder, biblabel,
###                        bibsort, and citesub, and is almost always
###                        in need of additional manual repair because
###                        of the poor quality of library catalog
###                        data.
###
###                        The definition of USMARC fields can be
###                        found at
###
###                            http://www.loc.gov/marc/bibliographic/ecbdhome.html
###
###                        Usage:
###                            awk -f usmarc-to-bibtex.awk MARC-file(s) >BibTeX-file
###
###                        This program is designed to be used with a
###                        wrapper shell script, cattobib.sh, that
###                        provides command-line options, default
###                        Z39.50 catalog names and short alias for
###                        them, and support for initialization files
###                        to define further aliases and catalog
###                        names.  The essential library catalog
###                        access is provided by the expect and
###                        yaz-client tools that are not part of the
###                        cattobib package, and must be downloaded in
###                        binary form, or compiled from source code,
###                        and installed somewhere on the search path.
###
###                        The checksum field above contains a CRC-16
###                        checksum as the first value, followed by the
###                        equivalent of the standard UNIX wc (word
###                        count) utility output of lines, words, and
###                        characters.  This is produced by Robert
###                        Solovay's checksum utility.",
###  }
### ====================================================================
###
### ====================================================================
### /u/sy/beebe/tex/bib/usmarc-to-bibtex.awk, Mon Nov 15 10:26:43 2004
### Edit by Nelson H. F. Beebe <beebe@math.utah.edu>
### Original version 0.00.
###
### A typical entry retrieved by ISBN looks like this:
###
###	035    $9 (DLC)   86021423
###	906    $a 7 $b cbc $c orignew $d 1 $e ocip $f 19 $g y-gencatlg
###	010    $a    86021423
###	020    $a 0070431523 (pbk.) : $c $19.95
###	040    $a DLC $c DLC $d DLC
###	050 00 $a QA76.76.O63 $b M68 1987
###	082 00 $a 005.4/465 $2 19
###	100 1  $a Morgan, Rachel.
###	245 10 $a Introducing UNIX System V / $c Rachel Morgan and Henry McGilton.
###	260    $a New York : $b McGraw-Hill, $c c1987.
###	300    $a xxv, 612 p. : $b ill. ; $c 23 cm.
###	500    $a Includes index.
###	630 00 $a UNIX System V (Computer file)
###	700 1  $a McGilton, Henry.
###	991    $b c-GenColl $h QA76.76.O63 $i M68 1987 $p 00034708459 $t Copy 1 $w BOOKS
###
### The same entry retrieved by the second author looks like this:
###
###	001 3751753
###	005 19910522102432.4
###	008 821022s1983    nyua          001 0 eng
###	035    $9 (DLC)   82021640
###	906    $a 7 $b cbc $c orignew $d 1 $e ocip $f 19 $g y-gencatlg
###	010    $a    82021640
###	020    $a 0070450013 (pbk.) : $c $18.95
###	040    $a DLC $c DLC $d DLC
###	050 00 $a QA76.8.U65 $b M38 1983
###	082 00 $a 001.64/4 $2 19
###	100 1  $a McGilton, Henry.
###	245 10 $a Introducing the UNIX system / $c Henry McGilton, Rachel Morgan.
###	260    $a New York : $b McGraw-Hill, $c c1983.
###	300    $a xix, 556 p. : $b ill. ; $c 23 cm.
###	440  0 $a McGraw-Hill software series for computer professionals
###	440  0 $a Byte books
###	504    $a Bibliography: p. 541-544.
###	500    $a Includes index.
###	630 00 $a UNIX (Computer file)
###	700 1  $a Morgan, Rachel.
###	991    $b c-GenColl $h QA76.8.U65 $i M38 1983 $p 00003514328 $t Copy 1 $w BOOKS
###
### Notice that the 100 and 700 records do not correctly identify the
### primary and secondary authors in the search by second author; only
### the 245.$c record does.
###
### [16-Sep-2013] -- Version 0.07 with many minor tweaks to field
###                  handling, and support for command-line variable
###                  USMARC=(yes|no) to control whether unrecognized
###                  USMARC fields are reported in the output.
### [21-Sep-2011] -- many small tweaks to improve formatting of data
###		     irregularities found in many different catalogs,
###		     including repair of unbalanced braced in strings
### [25-Mar-2005] -- add support for MARC 090 and extend fix_LCCN()
### [15-Nov-2004] -- original version
### ====================================================================

BEGIN		{ initialize() }

		{ gsub("[\r]+$", "") }		# reduce CR LF to LF

		{ gsub("\r", "") }		# remove all remaining spurious CR

		{ gsub("[&]apos;", "'") }	# appears often in carmin.sudoc.abes.fr entries

/^[ \t]*$/	{ print_entry(); next }

/^nextResultSetPosition/	{ print_entry(); next }

$0 ~ "Z> +open +"		{ Bibsource = $3 }

$1 == "001"	{ print_entry(); next }

$1 == "020"	{ USMARC_020($0); next }
$1 == "021"	{ USMARC_021($0); next }
$1 == "022"	{ USMARC_022($0); next }
$1 == "030"	{ USMARC_030($0); next }
$1 == "050"	{ USMARC_050($0); next }
$1 == "084"	{ USMARC_084($0); next }
$1 == "090"	{ USMARC_090($0); next }
$1 == "097"	{ USMARC_097($0); next }
$1 == "100"	{ USMARC_100($0); next }
$1 == "110"	{ USMARC_110($0); next }
$1 == "111"	{ USMARC_111($0); next }
$1 == "130"	{ USMARC_130($0); next }
$1 == "210"	{ USMARC_210($0); next }
$1 == "222"	{ USMARC_222($0); next }
$1 == "245"	{ USMARC_245($0); next }
$1 == "250"	{ USMARC_250($0); next }
$1 == "260"	{ USMARC_260($0); next }
$1 == "300"	{ USMARC_300($0); next }
$1 == "440"	{ USMARC_440($0); next }
$1 == "490"	{ USMARC_490($0); next }
$1 == "500"	{ USMARC_500($0); next }
$1 == "505"	{ USMARC_505($0); next }
$1 == "520"	{ USMARC_520($0); next }
$1 == "600"	{ USMARC_600($0); next }
$1 == "630"	{ USMARC_630($0); next }
$1 == "650"	{ USMARC_650($0); next }
$1 == "651"	{ USMARC_651($0); next }
$1 == "690"	{ USMARC_690($0); next }
$1 == "700"	{ USMARC_700($0); next }
$1 == "740"	{ USMARC_740($0); next }
$1 == "745"	{ USMARC_745($0); next }
$1 == "776"	{ USMARC_776($0); next }
$1 == "852"	{ USMARC_852($0); next }
$1 == "856"	{ USMARC_856($0); next }
$1 == "859"	{ USMARC_859($0); next }

### We explicitly ignore these uninteresting keys:

$1 == "005"	{ next }
$1 == "008"	{ next }
$1 == "010"	{ next }
$1 == "015"	{ next }
$1 == "035"	{ next }
$1 == "040"	{ next }
$1 == "041"	{ next }
$1 == "042"	{ next }
$1 == "082"	{ next }
$1 == "240"	{ next }
$1 == "504"	{ next }
$1 == "800"	{ next }
$1 == "906"	{ next }
$1 == "920"	{ next }
$1 == "922"	{ next }
$1 == "925"	{ next }
$1 == "955"	{ next }
$1 == "991"	{ next }

($0 ~ "^[ \t]")	{   # we have a continuation of the last USMARC record
		    if ($0 ~ "^[ \t]+$")
			;	# ignore blank lines
		    else if (Last_Key in Suffix) # continuation lines are always joined by space
		    {
			sub(/[.;] *$/,"",Entry[Last_Key]) # remove old separator
			append(Last_Key, trim($0), " ")
		    }
		    else
		    {
			unrecognized($1,$0 " DEBUG: Last_Key = [" Last_Key "]")
			Last_Key = ""
		    }
		    next
		}

		{ unrecognized($1,$0); Last_Key = ""; next }

END		{ terminate() }

# ========================================================================


function append(key,v,separator)
{
    if (key in Entry)
	set(key,Entry[key] separator v)
    else
	set(key,v)
}


function append_suffix(key,v)
{
    ## print "%%% DEBUG: append_suffix(): key = [" key "]  v = [" v "]"

    append(key, v, Suffix[key])
    Last_Key = key		# global value need to handle continuation lines
}


function author_editor(    result)
{
    if ("author" in Entry)
	result = Entry["author"]
    else if ("editor" in Entry)
	result = Entry["editor"]
    else
	result = Entry["author"] = "Anonymous"

    return (result)
}


function capitalize(s, k,n,parts)
{
    if (is_corporate_name(s))
	return (s)		# leave corporate names alone

    if (s ~ /[A-Z][A-Z]/)	# Reduce HEARN, A. C. to Hearn, A. C.
    {				# but leave McMahon, L. E. unchanged
	n = split(s,parts," ")

	for (k = 1; k <= n; ++k)
	{
	    s = (k == 1) ? \
		capitalize_word(parts[k]) : \
		s " " capitalize_word(parts[k])
	}
    }

    return (s)
}


function capitalize_word(s, k)
{
    for (k = 1; k <= length(s); ++k)
    {
	## if (isupper(substr(s,k,1)) && (k > 1) && isletter(substr(s,k-1,1)))
	## [14-May-1996]: Change isletter() above to isupper(), so that
	## we do not convert "McLean" to "Mclean".

	if (isupper(substr(s,k,1)) && (k > 1) && isupper(substr(s,k-1,1)))
	    s = substr(s,1,k-1) tolower(substr(s,k,1)) substr(s,k+1)
    }

    return (s)
}


function check_CODEN(the_CODEN,    checksum)
{
    ## CODEN values are 6-character strings from the set [A-Z0-9],
    ## with a check digit stored in the 6th position given by
    ##
    ##     (11*N1 + 7*N2 + 5*N3 + 3*N4 + 1*N5) mod 34 == X
    ##
    ## where the Nk are 1..26 for A..Z, and 27..36 for 1..9..0.
    ## However, the checksum X (in 0..33) is represented by the
    ## corresponding character in the different 34-character range
    ## [9A-Z2-8], which excludes digits 0 and 1 to avoid confusion
    ## with letters O and I.

    checksum = 0
    checksum += 11 * CODEN_Digit(the_CODEN, 1)
    checksum +=  7 * CODEN_Digit(the_CODEN, 2)
    checksum +=  5 * CODEN_Digit(the_CODEN, 3)
    checksum +=  3 * CODEN_Digit(the_CODEN, 4)
    checksum +=  1 * CODEN_Digit(the_CODEN, 5)

    if (int(checksum % 34) != CODEN_Check_Digit(the_CODEN, 6))
	warning("invalid CODEN: [" the_CODEN "]")
}


function check_CODEN_list(CODEN_list,    k,n,parts)
{
    CODEN_list = toupper(CODEN_list)
    n = split(CODEN_list, parts, "[,; ]+")

    for (k = 1; k <= n; ++k)
	check_CODEN(parts[k])
}


function check_ISBN(the_ISBN,    checksum,isbn,k)
{
    ## ISBN-10 values are strings containing nine characters from
    ## [0-9] and a tenth check digit in [0-9X], ignoring hyphens and spaces.
    ## The check digits is valid if
    ##	(sum(k=1:9) digit(k) * k) mod 11 == digit(10)
    ## where X as a digit has value 10.

    isbn = the_ISBN
    gsub("[- ]", "", isbn)

    checksum = 0

    for (k = 1; k <= 9; ++k)
	checksum += ISBN_Digit(isbn, k) * k

    if (int(checksum % 11) != ISBN_Digit(isbn, 10))
	warning("invalid ISBN: [" the_ISBN "]")
}


function check_ISBN_list(ISBN_list,    k,n,parts)
{
    ISBN_list = toupper(ISBN_list)
    n = split(ISBN_list, parts, "[,; ]+")

    for (k = 1; k <= n; ++k)
    {
	if (parts[k] !~ "^([^()]*)$")	# ignored parenthesized comments
	    check_ISBN(parts[k])
    }
}


function check_ISBN_13(the_ISBN_13,    checkdigit,checksum,isbn,k,remainder)
{
    ## The conversion from ISBN-10 to ISBN-13 is the same process
    ## currently used to encode the ISBN-10 into the 13-digit
    ## Bookland EAN:
    ##
    ## Step 1: Drop the check digit (the last digit) from your
    ## existing ISBN-10.
    ##
    ## For example - Your ISBN-10 is: 0-940016-61-3
    ## Dropping the check digit gives you a 9-digit number:
    ## 0-940016-61
    ##
    ## Step 2: Add the prefix "978" to the beginning of your 9-digit number.
    ##
    ## So, 0-940016-61 becomes 978-0-940016-61
    ##
    ## Step 3: Recalculate your check digit using the modulus 10
    ## check digit routine. This is the current routine used to
    ## calculate the check digit for the Bookland EAN.
    ##
    ##     EAN =                   9    7   8   0    9    4    0   0   1    6    6    1
    ##     Weighting Factors       1    3   1   3    1    3    1   3   1    3    1    3
    ##     Values (product)        9 + 21 + 8 + 0 +  9 + 12 +  0 + 0 + 1 + 18 +  6 +  3 +
    ##
    ## Sum of the products=87
    ## Divide the sum by the modulus (always 10)
    ## Quotient=10 with a remainder of 7
    ## The modulus (10) minus the remainder equals the check digit,
    ## with one exception.  If the remainder from Step 2 is 0, the
    ## check digit is 0.  In this case, however: 10-7=3.
    ## ISBN-13 check digit = 3
    ##
    ## Your ISBN-13 is: 978-0-940016-61-3

    isbn = the_ISBN_13
    gsub("[- ]", "", isbn)

    checksum = 0

    for (k = 1; k <= 12; ++k)
	checksum += ISBN_Digit(isbn, k) * ((int(k % 2) == 1) ? 1 : 3)

    remainder = int(checksum % 10)
    checkdigit = (remainder == 0) ? 0 : (10 - remainder)

    if (checkdigit != ISBN_Digit(isbn, 13))
	warning("invalid ISBN-13: [" the_ISBN_13 "]")
}


function check_ISBN_13_list(ISBN_13_list,    k,n,parts)
{
    ISBN_13_list = toupper(ISBN_13_list)
    n = split(ISBN_13_list, parts, "[,; ]+")

    for (k = 1; k <= n; ++k)
    {
	if (parts[k] !~ "^[(][^()]*[)]$")	# ignored parenthesized comments
	    check_ISBN_13(parts[k])
    }
}


function check_ISSN(the_ISSN,    checksum,issn,k)
{
    ## ISSN numbers are 8-character values from the set [0-9Xx], with
    ## a checksum given by
    ##
    ##	(sum(k=1:7) digit(k) * (k+2)) mod 11 == digit(8)
    ##
    ## where digits have their normal value, X (or x) as a digit has
    ## value 10, and spaces and hyphens are ignored.  The sum is
    ## bounded from above by 10*(3 + 4 + ... + 9) = 420.

    sub("-", "", the_ISSN)
    issn = toupper(the_ISSN)

    checksum = 0

    for (k = 1; k <= 7; ++k)
	checksum += ISSN_Digit(issn, k) * (k + 2)

    if (int(checksum % 11) != ISSN_Digit(issn, 8))
	warning("invalid ISSN: [" the_ISSN "]")
}


function check_ISSN_list(ISSN_list,    k,n,parts)
{
    ISSN_list = toupper(ISSN_list)
    n = split(ISSN_list, parts, "[,; ]+")

    for (k = 1; k <= n; ++k)
    {
	if ( (parts[k] != "(PRINT)") && (parts[k] != "(ELECTRONIC)") )
	    check_ISSN(parts[k])
    }
}


function check_for_proceedings(    s)
{
    if (("title" in Entry) && !("booktitle" in Entry))
    {
	s = tolower(Entry["title"] " " \
		    (("note" in Entry) ? Entry["note"] : "") \
		    (("USMARC-711" in Entry) ? Entry["USMARC-711"] : ""))

	if (s ~ "conference|congress|proceedings|school|symposium|workshop")
	{
	    Entry["booktitle"] = Entry["title"]
	    set("entrytype","Proceedings")

	    if (("author" in Entry) && !("editor" in Entry))
	    {
		Entry["editor"] = Entry["author"]
		delete Entry["author"]
	    }
	}
    }
}


function citation_label( author,abbrev,k,n,parts,t,tag)
{
    author = author_editor()

    if (substr(author,1,1) == "{")
    {				# have corporate author
	split(author,parts," ")	# "{SPSS, Inc."}
	t = substr(parts[1],2)	# "SPSS,"
	gsub(/,/,"",t)		# reduce to "SPSS"
    }
    else
	t = capitalize(author)
    gsub(/ and /, ",", t)		# for uniform field separators
    split(t,parts,",")	# careful: "De La Beaujardiere, Jean-M" has spaces in name
    gsub(/[^-A-Za-z'\055]/,"",parts[1])	# strip non-(letters,apostrophe)

    sub(/Jr$/,"",parts[1])	# remove any Jr suffix
    tag = parts[1] ":" Entry["year"] ":"

    ## We do a second strip here, because some entries for the year
    ## can have spaces and other words in them, and because we do
    ## not want apostrophes in the tag (bibxxx utilities complain).
    gsub(/[^-A-Za-z0-9:\055]/,"",tag)	# strip non-(letters,digits,colon))

    n = split(Entry["title"],parts," ")
    abbrev = ""

    for (k = 1; (k <= n) && (length(abbrev) < 3); ++k)
    {
	gsub(/[^A-Za-z0-9\'\055]/,"",parts[k]) # strip non-(letters,digits,apostrophe)

	if (isletter(substr(parts[k],1,1)))
	{
	    parts[k] = tolower(parts[k])

	    if (!(parts[k] in Ignore))
		abbrev = abbrev toupper(substr(parts[k],1,1))
	}
    }

    return (unique_citation_label(tag abbrev))
}


function clean_author_editor(s)
{
    ## Typical values:
    ## Smith, David, 1948 June 26-
    ## Danuloff, Craig, 1963-
    ## Dayton, Linnea

    gsub(/19[0-9][0-9]-? *([A-Z][a-z]+ [0-9]+-?)/,"",s)

    return (s)
}


function clean_entry()
{
    if ("author" in entry)
	set("author",clean_author_editor(Entry["author"]))

    if ("editor" in Entry)
	set("editor",clean_author_editor(Entry["editor"]))

    if ("title" in Entry)
    {
	gsub(/ *:/,":",Entry["title"])		# remove space before colons
	gsub(/ *\/[^\/]*$/,"",Entry["title"])	# strip author name(s) from title
	gsub("[ ]+[.][ ]+", " ", Entry["title"]) # strip isolated periods preceding subtitles
	gsub("[.]$", "", Entry["title"])	# strip trailing period
	gsub("}-{", "-", Entry["title"])	# e.g., reduce {S}-{PLUS} to {S-PLUS}

	if ("booktitle" in Entry)
	    Entry["booktitle"] = Entry["title"]
    }

    if ("edition" in Entry)
    {
	sub(/ ed[.]?$/,"",Entry["edition"])

	if (Entry["edition"] == "1st")
	    delete Entry["edition"]
	else if (Entry["edition"] ~ "1[.] Aufl[.]")
	    delete Entry["edition"]
	else if (Entry["edition"] == "1. print.")
	    delete Entry["edition"]
	else if (Entry["edition"] == "1.")
	    delete Entry["edition"]
	else if (Entry["edition"] == "1..")
	    delete Entry["edition"]
	else if (Entry["edition"] == "2nd")
	    set("edition","Second")
	else if (Entry["edition"] == "3rd")
	    set("edition","Third")
	else if (Entry["edition"] == "4th")
	    set("edition","Fourth")
	else if (Entry["edition"] == "5th")
	    set("edition","Fifth")
	else if (Entry["edition"] == "6th")
	    set("edition","Sixth")
	else if (Entry["edition"] == "7th")
	    set("edition","Seventh")
	else if (Entry["edition"] == "8th")
	    set("edition","Eighth")
	else if (Entry["edition"] == "9th")
	    set("edition","Nineth")
	else if (Entry["edition"] == "10th")
	    set("edition","Tenth")
	else if (Entry["edition"] == "Rev.")
	    set("edition","Revised")
    }

    if ("ISBN" in Entry)
    {
	gsub(/ISBN */, "", Entry["ISBN"])
	clean_ISBN()
	Entry["ISBN"] = fix_ISBN(Entry["ISBN"])
    }

    if ( ("ISBN-13" in Entry) &&  ("ISBN" in Entry) )
	hyphenate_isbn_13()

    if (("note" in Entry) && (substr(Entry["note"],length(Entry["note"]),1) != "."))
	Entry["note"] = Entry["note"] "."

    if ("price" in Entry)	# NB: must come after ISBN processing
    {
	gsub(/ *U[.]?S[.]?A[.]? */," ",Entry["price"])
	gsub(/ *,/,",",Entry["price"])
    }

    if ("address" in Entry)
	Entry["address"] = fix_address(Entry["address"])

    fix_annote()

    if ("author-dates" in Entry)
    {
	if (Entry["author-dates"] ~ "-- *$") # do not record dates for living authors
		delete Entry["author-dates"]
    }

    if ("note" in Entry)
	Entry["note"] = fix_note(Entry["note"])

    if ("pages" in Entry)
	Entry["pages"] = fix_pages(Entry["pages"])

    Entry["LCCN"] = ("LCCN" in Entry) ? fix_LCCN(Entry["LCCN"]) : "????"

    if ("abstract" in Entry)
	Entry["abstract"] = fix_abstract(Entry["abstract"])

    if ("series" in Entry)
	gsub("; *; *", "; ", Entry["series"])

    if ("subject" in Entry)
	Entry["subject"] = fix_subject(Entry["subject"])

    if ("URL" in Entry)
	Entry["URL"] = fix_URL(Entry["URL"])

    if ("volume" in Entry)
	Entry["volume"] = fix_volume(Entry["volume"])

    if ("year" in Entry)
	Entry["year"] = fix_year(Entry["year"])

    if (Bibsource != "")
	Entry["bibsource"] = Bibsource
}


function clean_ISBN(    ISBN, parts)
{
    ## Typical values
    ##	"0070240493 (H) : US\$29.95, 0070240507 (P) : US\$19.95"
    ##   "0442019963 (Van Nostrand Reinhold), 1850321418 (ITP UK)"
    ##   "0125184085 (acid-free paper)"
    ##   "0125531303 (alk. paper), 0125531311 (CD-Rom : alk. paper)"
    ##   "0127378405 (alk. paper), 0127378413 (CD-ROM)"
    ##   "0201633957 (alk. paper)"
    ##   "0262112078 (alk. paper), 026261118X (pbk. : alk. paper)"
    ##   "0471038571 (alk. paper : pbk.) : US\$19.95"
    ##   "0838976972 (alk. paper) : US\$18.00 (LITA mbr. US\$16.20)"
    ##   "0887369332 (alk. paper : pbk.) : US\$30.00 (US\$37.50 Can. : L21.00 UK)"
    ##   "0471130885 (cloth : alk. paper)"
    ##   "0841232237 (clothbound), 0841232245 (pbk.)"
    ##   "0078811821 (cover), 0078821821: US\$34.95"
    ##   "0079121004 (p)"
    ##   "0471113360 (paper : acid-free paper) : US\$16.95"
    ##   "0471128430 (paper : alk. paper)"
    ##   "0712332952 (pbk)"
    ##   "0471126942 (pbk. : acid-free paper)"
    ##   "047114178X (pbk. : alk. paper)"
    ##   "007024054X (pbk.) : US\$19.95"
    ##   "020140978X (pbk.) US\$26.95 (US\$34.95 Can.)"
    ##   "2707125024 (pbk.) : 140FF"
    ##   "0192627058 (pbk.)"
    ##   "0131515659 (recycled paper)"
    ##   "0387945547 (softcover : alk. paper)"
    ##   "0446671797 (trade pbk.)"
    ##   "088736943X (v. 1 : alk. paper) : US\$7.00, 0887369448 (v. 2 : alk. paper) : US\$7.00, 0887369456 (v. 3 : alk. paper) : US\$7.00, 0887369464 (v. 4 : alk. paper) : US\$7.00, 0887369472 (v. 5 : alk. paper) : US\$7.00, 0887369480 (v. 6 : alk. paper) : US\$7.00"
    ##   "012553132X, 0125531338 (CD-ROM)"
    ##   "1556158858, 1556158181"
    ##   "0070240558 : US\$16.95"
    ##   "0070670188 : US\$45.00, 0070670196 (pbk.) : US\$29.95"
    ##   "0077079051 : L22.95"
    ##   "013185612X : US\$29.95 (US\$39.95 Can.)"
    ##   "0201410001 : US\$17.95 (Can. US\$22.95)"
    ##   "0201627078 : US\$14.95 (US\$18.95 Can.)"
    ##   "0672305305 : US\$25.00 (US\$34.95 CAN)"
    ##   "1562054376 : US\$35.00 (US\$47.95 Can.) (L32.49 UK)"
    ##   "156761521X : US\$12.99"
    ##   "0201407663 US\$12.95 (US\$16.95 Can.)"
    ##   "0789701375 US\$39.99 (US\$53.99 Can.)"

    ISBN = entry["ISBN"]

    ## print "%% DEBUG 1: clean_ISBN(): ISBN = [" ISBN "]"

    gsub(/[$]/,"US\\$",ISBN)

    gsub(/ +,/,",",ISBN)
    gsub(/\(v[.] /,"(vol. ",ISBN)
    gsub(/acid-free paper/,"",ISBN)
    gsub(/alk[.] paper/,"",ISBN)
    gsub(/mbr[.]/,"members",ISBN)
    gsub(/pbk[.]/,"paperback",ISBN)
    gsub(/pbk/,"paperback",ISBN)
    gsub(/\( *\)/,"",ISBN)
    gsub(/\([pP]\)/,"(paperback)",ISBN)
    gsub(/\([hH]\)/,"(hardcover)",ISBN)
    gsub(/ *: *\)/,")",ISBN)
    gsub(/\( *: */,"(",ISBN)
    gsub(/CD-Rom/,"CD-ROM",ISBN)
    gsub(/C[Aa][Nn][Aa][Dd][Aa] */,"Can.",ISBN)
    gsub(/C[Aa][Nn][.]? *US/,"CDN",ISBN)

    gsub(/[ :]+$/,"",ISBN)
    gsub(/[ ]+/," ",ISBN)

    ## print "%% DEBUG 2: clean_ISBN(): ISBN = [" ISBN "]"

    while (match(ISBN,/US\\[$][0-9.]+ C[Aa][Nn][.]/))
    {
	ISBN = substr(ISBN,1,RSTART-1) "CDN" substr(ISBN,RSTART+2,RLENGTH-7) \
	  substr(ISBN,RSTART+RLENGTH)
    }

    while (match(ISBN,/US\\[$][0-9.]+ C[Aa][Nn]/))
    {
	ISBN = substr(ISBN,1,RSTART-1) "CDN" substr(ISBN,RSTART+2,RLENGTH-6) \
	  substr(ISBN,RSTART+RLENGTH)
    }

    while (match(ISBN,/L[0-9.]+/))
    {
	ISBN = substr(ISBN,1,RSTART-1) "UK\\pounds" substr(ISBN,RSTART+1,RLENGTH-1) \
	  substr(ISBN,RSTART+RLENGTH)
    }

    while (match(ISBN,/[0-9] : UK/))
    {
	ISBN = substr(ISBN,1,RSTART) "," substr(ISBN,RSTART+3)
    }

    while (match(ISBN,/\(CDN\\\$[0-9.]+\)/))
    {
	ISBN = substr(ISBN,1,RSTART-1) ", " substr(ISBN,RSTART+1,RLENGTH-2)
    }

    while (match(ISBN,/\(UK\\pounds[0-9.]+\)/))
    {
	ISBN = substr(ISBN,1,RSTART-1) ", " substr(ISBN,RSTART+1,RLENGTH-2)
    }

    ## print "%% DEBUG 3: clean_ISBN(): ISBN = [" ISBN "]"

    gsub(/[ ]+,/,",",ISBN)
    gsub(/[ :]+$/,"",ISBN)
    gsub(/[ ]+/," ",ISBN)

    ## print "%% DEBUG 4: clean_ISBN(): ISBN = [" ISBN "]"

    if ((index(ISBN,":") == 0) && \
	match(ISBN,/[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9xX],?( \([^)]*\))? *[A-Z]/))
	ISBN = substr(ISBN,1,RSTART + RLENGTH - 2) ": " substr(ISBN,RSTART + RLENGTH - 1)

    ## print "%% DEBUG 5: clean_ISBN(): ISBN = [" ISBN "]"

    if (split(ISBN,parts,":") == 2)
    {
	ISBN = parts[1]

	gsub(/^ +/,"",parts[2])
	gsub(/[ ]+/," ",parts[2])
	entry["price"] = entry["price"] ", " parts[2]
	sub(/^, */,"",entry["price"])
    }

    ## print "%% DEBUG 6: clean_ISBN(): ISBN = [" ISBN "]"

    gsub(/[ ,]+$/,"",ISBN)

    gsub("[(] *cloth,* *[)], *", "(cloth)", ISBN)
    gsub("[(] pbk.,* *[)], *", "(paperback)", ISBN)
    gsub("^[(][a-z]+[)],? *", "", ISBN)

    ## print "%% DEBUG 7: clean_ISBN(): ISBN = [" ISBN "]"

    entry["ISBN"] = fix_ISBN(ISBN)
}


function clear_array(array, key)
{
    for (key in array)
	delete array[key]
}


function CODEN_Check_Digit(the_CODEN,n)
{

    return (index("9ABCDEFGHIJKLMNOPQRSTUVWXYZ2345678", substr(the_CODEN, n, 1)) - 1)
}


function CODEN_Digit(the_CODEN,n)
{

    return (index("ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890", substr(the_CODEN, n, 1)))
}


function delete_match(s)
{
    ## After a match() call on s, return s after deleting the match

    return (substr(s,1,RSTART-1) substr(s,RSTART+RLENGTH))
}


function enquote(v)
{
    if (substr(v,1,1) == "\"")	# enquote leading quote
	v = "{\377}" substr(v,2)

    while (match(v,/[^\\]\"/))
    {				# brace unbackslashed quotes
	v = substr(v,1,RSTART) "{\377}" substr(v,RSTART+2)
    }

    gsub(/\377/,"\"",v)

    gsub(/&/," and ",v)		# replace ampersands
    gsub(/[ \t][ \t]+/," ",v)	# eliminate redundant spaces
    gsub(/[%]/,"\\%",v)		# protect quotes
    gsub(/[#]/,"\\#",v)		# protect sharps
    gsub(/[_]/,"\\_",v)		# protect underscores
    gsub(/[\^]/,"\\^",v)	# protect carets

    if (v ~ /[0-9][$][0-9]/)
	gsub(/[$]/,"--",v)	# June 7$10 -> June 7--10
#    else if (v ~ /[$][0-9]+/)
#	gsub(/[$]/,"US\\$",v)	# $27 -> US\$27 and $27.95 -> US\$27.95

    return (v)
}


function entry_name( )
{
    return (("entrytype" in Entry) ? \
	Entry["entrytype"] : "????UNKNOWN-TYPE-????")
}


function fix_abstract(s)
{
    ## Remove duplicate MARC-record joining dots (from Suffix["abstract"])
    gsub("[.][.][.]* ", ". ", s)
    sub("[.][.][.]*$",  ".",  s)

    return (s)
}


function fix_address(s)
{
    s = trim(squeeze_blanks(s))

    ## print "%% DEBUG: enter fix_address(\"" s "\")" > "/dev/tty"

    if (match(s, "^\\[.*\\]$")) # reduce "[Philadelphia]" to "Philadelphia"
	s = substr(s, RSTART + 1, RLENGTH - 2)

    sub(" *[:;/] *$", "", s)

    ## Major publishers are in only a few states
    sub(" Cal[.]$",	" CA, USA",	s)
    sub(" Calif[.]$",	" CA, USA",	s)
    sub(" Mass[.]$",	" MA, USA",	s)
    sub(" Minn[.]$",	" MN, USA",	s)
    sub(" N[.]J[.]$",	" NJ, USA",	s)
    sub(" N[.]Y[.]$",	" NY, USA",	s)
    sub("Washington, D[.]C[.]$",	"Washington, DC, USA",	s)

    ## Major publishers are in only a few cities
    sub("^Boston$",		"Boston, MA, USA",	s)
    sub("^Chicago$",		"Chicago, IL, USA",	s)
    sub("^New York$",		"New York, NY, USA",	s)
    sub("^Philadelphia$",	"Philadelphia, PA, USA",s)
    sub("^San Francisco$",	"San Francisco, CA, USA",	s)
    sub("^Washington$",		"Washington, DC, USA",	s)

    ## Fix a few non-US publisher addresses
    sub("^Amsterdam$",	"Amsterdam, The Netherlands",	s)
    sub("^Berlin$",	"Berlin, Germany",	s)
    sub("^Brussels$",	"Brussels, Belgium",	s)
    sub("^Bruxelles$",	"Bruxelles, Belgium",	s)
    sub("^Copenhagen$",	"Copenhagen, Denmark",	s)
    sub("^Edinburgh$",	"Edinburgh, Scotland",	s)
    sub("^Heidelberg$",	"Heidelberg, Germany",	s)
    sub("^Lisbon$",	"Lisbon, Portugal",	s)
    sub("^London$",	"London, UK",		s)
    sub("^Madrid$",	"Madrid, Spain",	s)
    sub("^Montreal$",	"Montr{\\'e}al, Qu{\\'e}bec, Canada",	s)
    sub("^Montr{\\\\'e}al$",	"Montr{\\'e}al, Qu{\\'e}bec, Canada",	s)
    sub("^Mnchen$",	"M{\\\"u}nchen, Germany",	s)
    sub("^Oslo$",	"Oslo, Norway",		s)
    sub("^Oxford$",	"Oxford, UK",		s)
    sub("^Paris$",	"Paris, France",	s)
    sub("^Philadelpha",	"Philadelpha, PA, USA",	s)
    sub("^Roma$",	"Roma, Italy",		s)
    sub("^Stockholm$",	"Stockholm, Sweden",	s)
    sub("^Stuttgart$",	"Stuttgart, Germany",	s)
    sub("^Toronto$",	"Toronto, ON, Canada",	s)
    sub("^Toronto, Canada$",	"Toronto, ON, Canada",	s)
    sub("^Warszawa$",	"Warszawa, Poland",	s)

    ## print "%% DEBUG: leave fix_address(\"" s "\")"

    return (s)
}


function fix_annote( n,parts,e,s)
{
    ## Remove some unwanted phrases in annote fields.

    if ("annote" in Entry)
    {
	s = Entry["annote"]
	gsub(/Includes bibliograph?ic(al)? references *(and index(es)?)? *(\([^\)]*\))?[.]?/,"",s)
	gsub(/Includes bibliograph?ic(al)? references *(p[.])? *(\([^\)]*\))? *(and index(es)?)?[.]?/,"",s)
	gsub(/Includes bibliograph?ic(al)? references *(and index(es)?)?[.]?/,"",s)
	gsub(/Includes bibliograph?(ies|y) *(and index(es)?)?[.]?/,"",s)
	gsub(/Includes glossar(y|ies) *(and index(es)?)?[.]?/,"",s)
	gsub(/Includes (author)? *index(es)?[.]?/,"",s)
	gsub(/Includes bibliograph?y[.]? *\([^\)]*\)[.]?/,"",s)
	gsub(/Includes bibliograph?(ic|ical|y) references[.]? *(\([^\)]*\))?(, *glossar(ies|y))? *(and *index(es)?)?[.]?/,"",s)
	gsub(/Includes bibliograph?(y|ies) \([^\)]*\) *(and index(es)?)[.]?*/,"",s)

	gsub(/Title from cover[.]?/,"",s)
	gsub(/Bibliograph?(y|ies) *: *(leaves|pages|pp[.]?|p[.]?) [0-9\-]*[.]?/,"",s)
	gsub(/Index *included[.]?/,"",s)
	gsub(/Index[.]?/,"",s)
	gsub(/ *and index[.]?/,"",s)

	if (!( ("month" in Entry) && ("day" in Entry) ))
	{
	    if (match(s,
		      /(January|February|March|April|May|June|July|August|September|October|November|December) *[123][0-9]?, *[12][0-9][0-9][0-9]/))
	    {
		n = split(substr(s,RSTART,RLENGTH),parts," ")

		if ((n == 3) && !("year" in Entry))
		    Entry["year"] = fix_year(parts[3])

		if ((n == 3) && (parts[3] == Entry["year"]))
		{
		    sub(/,$/,"",parts[2])
		    Entry["day"] = parts[2]
		    Entry["month"] = month_abbrev[tolower(parts[1])]
		}
	    }
	    else if (match(s,
		      /[123][0-9]? *(January|February|March|April|May|June|July|August|September|October|November|December), *[12][0-9][0-9][0-9]/))
	    {
		n = split(substr(s,RSTART,RLENGTH),parts," ")

		if ((n == 3) && !("year" in Entry))
		    Entry["year"] = fix_year(parts[3])

		if ((n == 3) && (parts[3] == Entry["year"]))
		{
		    sub(/,$/,"",parts[2])
		    Entry["day"] = parts[1]
		    Entry["month"] = month_abbrev[tolower(parts[2])]
		}
	    }
	}

	if (s ~ /^[ ]*$/)
	    delete Entry["annote"]
	else
	    set("annote",s)
    }
}


function fix_author_4(s)
{
    ## Reduce
    ##  "Robbins, Arnold. $4 aut."
    ##  "Robbins, Arnold. $0 (DE-601)081054718 $0 (DE-588a)121051005"
    ##  "Lichtenberg, Kathrin. $4 trl"
    ## to
    ##  "Robbins, Arnold."
    ##  "Robbins, Arnold."
    ##  "Lichtenberg, Kathrin (translator)"
    ## and return the result
    gsub(" *[$]4 edt[.]? *", " ", s)
    gsub(" *[$]4 aut[.]? *", " ", s)
    gsub(" *[$]4 trl[.]? *", " (translator) ", s)
    gsub(" *[$]0 .*$", "", s)

    return (s)
}


function fix_author_c(s)
{
    ## Reduce
    ##	"Scott, Walter,|cSir,|d1771-1832"
    ## to
    ##	"Scott, Walter, Sir,|d1771-1832"
    ## and return the result
    gsub("[|$]c"," ",s)

    return (s)
}


function fix_author_d(s)
{
    ## Reduce
    ##	"Scott, Walter,|cSir,|d1771-1832"
    ## to
    ##	"Scott, Walter,|cSir"
    ## save the dates in a separate entry,
    ## and return the result

    return (subvalue(s, "d", "author-dates"))
}


function fix_author_editor_confusion()
{
    ## Try to sort out confusion between author and editor values
    ## arising from the lack of a clear distinction between them in
    ## MARC records.  It is critical that NO EDITING is done on the
    ## values; any needed cleanup is handled later after a citation
    ## label has been constructed.

    if (("author" in Entry) && (Entry["author"] == ""))
	Entry["author"] = "Anonymous"
    else if (("editor" in Entry) && (Entry["editor"] == ""))
	Entry["editor"] = "Anonymous"

    if ((!("author" in Entry)) && (!("editor" in Entry)) )
	Entry["author"] = "Anonymous"

    if ( (!("editor" in Entry)) && ("USMARC-245-c" in Entry) )
    {
	if (Entry["USMARC-245-c"] ~ "[Ee]dited|[Ee]ditor")
	{
	    if ("author" in Entry)
	    {
		Entry["xxauthor"] = Entry["editor"] = Entry["author"]
		delete Entry["author"]
	    }

	    if ( ("xxauthor" in Entry) && !("xxeditor" in Entry) )
	    {
		Entry["xxeditor"] = Entry["xxauthor"]
		delete Entry["xxauthor"]
	    }
	}
    }
}


function fix_author_editor_names(s,    t, years)
{
    ## Reduce "Henry. McGilton and Rachel. Morgan" to
    ## "Henry McGilton and Rachel Morgan"
    ## and
    ## "Haenel, Christoph, $d 1939-" to "Haenel, Christoph"
    ## and
    ## "Lowdin, Per-Olov, $d 1916-2000 $4 oth" to "Lowdin, Per-Olov"

    ## print "%% DEBUG: enter fix_author_editor_names(\"" s "\")"

    t = s

    if (match(s, "^ *and +"))
	s = substr(s, RSTART + RLENGTH)

    gsub(" *[(][- 0-9]+[)][.]? *", " ", s)	# remove author/editor dates

    gsub(" +[.] +", " ", s)			# remove isolated stray periods

    s = squeeze_blanks(trim(s))

    if (match(s, "[a-z][.]$"))			# reduce "Mark Twain." to "Mark Twain"
	s = substr(s, 1, RSTART + RLENGTH - 2)

    if (s ~ " [JS]r$")				# restore period after final suffix
	s = (s ".")

    s = (s " ")

    if (match(s, "[$]d +[12][0-9][0-9][0-9]-+[^0-9]"))
	s = substr(s, 1, RSTART - 1) fix_author_editor_names(substr(s, RSTART + RLENGTH - 1))

    s = (s " ")

    if (match(s, "[$]d +[12][0-9][0-9][0-9]-[12][0-9][0-9][0-9]"))
	s = substr(s, 1, RSTART - 1) fix_author_editor_names(substr(s, RSTART + RLENGTH))

    s = (s " ")

    if (match(s, "[$]4 +[a-z]+"))
	s = substr(s, 1, RSTART - 1) fix_author_editor_names(substr(s, RSTART + RLENGTH))

    gsub("[ ]+[$][a-z][ ]+", " ", s)	# remove MARC subfield markers

    gsub(" +and +and +", " and ", s)
    gsub(", *[.]? *$", "", s)

    ## Remove editor suffixes
    gsub(", +ed[.]? ", " ", s)

    s = trim(squeeze_blanks(s))

    if (match(t, " [$]d [12][0-9][0-9][0-9] *- *[12][0-9][0-9][0-9] "))
    {
	years = substr(t, RSTART + 4, RLENGTH - 5)
	gsub(" ", "", years)
	gsub("-+", "--", years)
	append_suffix("author-dates", (s " (" years ")"))
    }

    ## print "%% DEBUG: leave fix_author_editor_names(): [" s "]"

    return (fix_author_4(s))
}


function fix_author_order()
{
    if ("author" in Entry)
	Entry["author"] = fix_names(Entry["author"])

    if ("xxauthor" in Entry)
    {
	Entry["xxauthor"] = fix_names(Entry["xxauthor"])

	if ( ("author" in Entry) && (Entry["author"] == Entry["xxauthor"]) )
	    delete Entry["xxauthor"]
    }
}


function fix_author_q(s)
{
    ## Reduce
    ##	"Wells, H. G.|q(Herbert George),|d1866-1946"
    ## to
    ##	"Wells, H. G. (Herbert George),|d1866-1946"
    ## and return the result
    gsub("[|$]q"," ",s)

    return (s)
}


function fix_brace_level(s,    c, k, level, n, t)
{
    ## Patch s, if needed, to ensure balanced braces, and return the result.
    ## Any additionally-supplied braces are identified with doubled question marks.

    if (match(s, "[{}]"))
    {
	level = 0
	n = split(s, c, "")	# NB: gawk extension: split into one-character parts
	t = ""

	for (k = 1; k <= n; ++k)
	{
	    if (c[k] == "{")
		level++
	    else if (c[k] == "}")
		level--

	    if (level < 0)
	    {
		t = t "{??"
		level = 0
	    }

	    t = (t c[k])
	}

	while (level > 0)
	{
	    t = (t "??}")
	    level--
	}

	s = t
    }

    gsub("\"", "{\"}", s)	# protect embedded quotes for BibTeX

    ## Patch commonly-seen accented letters
    gsub("\\\\{\"}a", "{\\\"a}", s)
    gsub("\\\\{\"}o", "{\\\"o}", s)
    gsub("\\\\{\"}u", "{\\\"u}", s)

    gsub("\\\\{\"}A", "{\\\"A}", s)
    gsub("\\\\{\"}O", "{\\\"O}", s)
    gsub("\\\\{\"}U", "{\\\"U}", s)

    ## Remove redundant brace level
    gsub("[{][{]\\\\\"a[}][}]", "{\\\"a}", s)
    gsub("[{][{]\\\\\"o[}][}]", "{\\\"o}", s)
    gsub("[{][{]\\\\\"u[}][}]", "{\\\"u}", s)

    gsub("[{][{]\\\\\"A[}][}]", "{\\\"A}", s)
    gsub("[{][{]\\\\\"O[}][}]", "{\\\"O}", s)
    gsub("[{][{]\\\\\"U[}][}]", "{\\\"U}", s)

    return (s)
}


function fix_editor_order()
{
    if ("editor" in Entry)
	Entry["editor"] = fix_names(Entry["editor"])

    if ("xxeditor" in Entry)
    {
	Entry["xxeditor"] = fix_names(Entry["xxeditor"])

	if ( ("editor" in Entry) && (Entry["editor"] == Entry["xxeditor"]) )
	    delete Entry["xxeditor"]
    }
}


function fix_ISBN(s,    s_org)
{
    ## Typical values:
    ##	"0070450013 (pbk.)"
    ##	"0201134470 :, 0201134489 (pbk)",
    ##	"3540665722 (softcover : acid-free paper)"

    s_org = s

    gsub("HB:", "(hardcover) ", s)
    gsub("PB:", "(paperback) ", s)
    gsub(" *:? *acid[- ]?free,? *paper", "", s)
    gsub(" *:? *alk[.]?,? *paper,?", "", s)
    gsub(" *[(] *[Hh]c[.]? *[)]", " (hardcover)", s)
    gsub(" *[(] *[Hh]bk?[.]? *[)]", " (hardcover)", s)
    gsub(" *[(] *[Pp]bk?[.]? *[)]", " (paperback)", s)
    gsub(" *[(] *[Pp]bk?[.]? *: *alk[.],? *paper[)]", " (paperback)", s)
    gsub(" *: *, *", ", ", s)
    gsub("[(] *[)]", "", s)
    gsub("^, *", "", s)

    gsub("^ *[(]e-book[)],* *",		"",	s)
    gsub("^ *[(]electronic[)],* *",	"",	s)
    gsub("^ *[(]hardback[)],* *",		"",	s)
    gsub("^ *[(]hardcover[)],* *",	"",	s)
    gsub("^ *[(]paperback[)],* *",	"",	s)

    ## carmin.sudoc.abes.fr has additional text like
    ## "( Edition imprimee)"
    ## as well as abbreviations for
    ## broch		paperback
    ## reli		hardcover

    gsub(" *[(] *Edition imprimee *[)] *", " (print) ", s)

    gsub(" *[(] *br *[)] *", " (paperback) ", s)
    gsub(" *[(] *broch *[)] *", " (paperback) ", s)

    gsub(" *[(] *rel *[)] *", " (hardcover) ", s)
    gsub(" *[(] *reli *[)] *", " (hardcover) ", s)

    ## German equivalents
    gsub(" *[(] *Kt[.]? *[)] *", " (paperback) ", s)
    gsub(" *[(] *Taschenbuch *[)] *", " (paperback) ", s)
    gsub(" *[(] *Tasch[a-z]*[.] *[)] *", " (paperback) ", s)
    gsub(" *[(] *brosch[a-z]*[.] *[)] *", " (paperback) ", s)

    ## Swedish equivalents: hftet, inbunden
    gsub(" *[(] *hft[.]? *[)] *", " (paperback) ", s)
    gsub(" *[(] *inb[.]? *[)] *", " (hardcover) ", s)

    ## Croatian (??) equivalents
    gsub(" *[(] *bro\\v *s[.]? *[)] *", " (paperback) ", s)
    gsub(" *[(] *vezano *[)] *", " (hardcover) ", s)

    gsub("[(]electronic bk.[)]", " (e-book) ", s)
    gsub("[(]elect. bk.[)]", " (e-book) ", s)
    gsub("[(]ebook[)]", " (e-book) ", s)

    gsub("[(]cased[)]", " (hardcover) ", s)

    gsub("[(]v[.] ", " (vol. ", s)

    ## print "%% DEBUG: fix_ISBN(): [" s_org "] -> [" s "]" > "/tmp/foo.dbg.isbn"

    gsub(" +, *", ", ", s)

    return (s)
}


function fix_LCCN(s,    k,key,lccns,n,parts)
{
    ## Typical values:
    ##	"QA76.76.O63 $b M68 1987"
    ##	"Z253 $b .U69 1993 090 $a Z253 $b .C57 1993 090 $a Z253 $b .C57 1993"
    gsub(" +090 +[$]a +", "; ", s)
    gsub(" +[$]b +", " ", s)

    n = split(s, parts, " *; *")

    for (k = 1; k <= n; ++k)		# construct array of unique LCCN values
	lccns[parts[k]]++

    s = ""

    for (key in lccns)			# reconstruct list of unique LCCN values
	s = (s == "") ? key : (s "; " key)

    gsub(" [*]*See *$", "", s)
    gsub("; x; *", "; ", s)
    gsub("; *Internet *", "", s)
    gsub(";*$", "", s)

    gsub("[$][abc]", " ", s)

    s = trim(squeeze_blanks(s))

    return (s)
}


function fix_name_order(s, has_junior,has_others,has_sir,junior,n,parts,t)
{
    ## Typical values (numbers indicate parts after splitting at commas)
    #
    ## Davis, Mitchell, Jr		-> Mitchell {Davis, Jr.}	123 -> 2 {1, 3}
    ## Murphy, Harry M. , Jr		-> Harry M. {Murphy, Jr.}	123 -> 2 {1, 3}
    ## Donnelly, William J., II		-> William J. {Donnelly, II}	123 -> 2 {1, 3}
    ## Reed, J. F., III			-> J. F. {Reed, III}		123 -> 2 {1, 3}
    #
    ## Walsh, John E.			-> John E. Walsh		12 -> 2 1
    ## Walsh, John E. (John Edward)	-> John E. (John Edward) Walsh	12 -> 2 1
    ## Beethoven, Ludwig von		-> Ludwig von Beethoven		12 -> 2 1
    #
    ## von Beethoven, Ludwig		-> Ludwig {von Beethoven}	12 -> 2 {1}
    ## Auf Der Heide, Friedhelm Meyer	-> Friedhelm Meyer {Auf Der Heide} 12 -> 2 {1}
    #
    ## This case requires special handling because the word order differs from the others:
    ## Beerbohm, Max, Sir		-> Sir Max Beerbohm
    #
    ## This one requires an extra brace, because part 1 contains a non-letter:
    ## Butterworth IV, John F		-> John F. {Butterworth IV}	12 -> 2{1}
    #
    ## This one requires special handling of Jr:
    ## Lanahan, Robert J. Jr		-> Robert J. {Lanahan, Jr.}	12 -> xxxx
    ## This case suggests that we always handle Jr specially.
    #
    ## Notice that people like Guy L. Steele Jr. who omit the comma
    ## before Jr must be coded in the library catalog like this:
    ## Steele Jr., Guy L.
    ## instead of
    ## Steele, Guy L., Jr.
    ## A search of the OCLC WorldCat database for this author showed that
    ## in every case, the Jr is omitted entirely.

    gsub(/[ ,]*$/,"",s)		# remove trailing space and comma
    gsub(/[.] +[)]/,".)",s)	# "(Stuart I. )" -> "(Stuart I.)"

    t = trim(s)

    has_sir = match(t,/[, ]Sir$/) || match(t,/[, ]Count$/) || match(t,/[, ]Lord$/)
    has_junior = match(t,/[, ][JS]r[.]?$/) || match(t,/[, ][IVX]$/)

    if (has_junior)
    {
	junior = substr(t,RSTART+1,RLENGTH-1)

	if (junior ~ /^[JS][r]$/) # supply Jr./Sr. abbreviation period
	    junior = junior "."

	t = substr(t,1,RSTART-1) # remove Jr
	sub(/[, ]*$/, "", t)	# remove trailing space and comma (but NOT periods)
    }

    has_others = match(t,/[, ]et al[.]?/)

    if (has_others)
	t = delete_match(t)

    n = split(t, parts, /, */)

    parts[1] = trim(parts[1])

    if (match(parts[1]," "))
	parts[1] = "{" parts[1] "}"

    if (is_corporate_name(s))
	t = "{" trim(s)	"}"	# only brace corporate authors
    else if (n > 3)
    {
	warning("Cannot reorder a personal name with more than 3 parts: [" sample(s) "]")
	t = trim(s)
    }
    else if (n == 3)
    {
	if (has_junior)
	    t = parts[2] " {" parts[1] ", " parts[3] ", " junior "}"
	else if (has_sir)
	    t = parts[3] " " parts[2] " " parts[1]
	else
	    t = parts[2] " {" parts[1] ", " parts[3] "}"
    }
    else if (n == 2)
    {
	if (has_junior)
	    t = parts[2] " {" parts[1] ", " junior "}"
	else
	    t = parts[2] " " parts[1]
    }
    else
	t = s

    if (has_others)
	t = t " and others"

    return (t)
}


function fix_names(s, k,n,person,t)
{
    ## print "%% DEBUG 1: fix_names() : s = [" squeeze_blanks(s) " ]"

    n = split(s, person, " and ")
    t = fix_name_order(person[1])

    for (k = 2; k <= n; ++k)
	t = t " and " fix_name_order(person[k])

    ## print "%% DEBUG 2: fix_names() : t = [" squeeze_blanks(t) " ]"

    return (t)
}


function fix_note(s)
{
    ## Typical values:
    ##     "Includes index."
    ##     "blah blah blah.--T.p. verso."

    ## print "%% DEBUG: enter fix_note(\"" s "\")"

    sub("Includes index[.] *", "", s)
    sub("Index[.] *$", "", s)
    sub("Selected bibliography[.] *", "", s)
    sub("-+T[.]p[.][ ]+verso.?", "", s)
    gsub("[.][.][.]", "\\ldots{}", s)
    gsub("[.][.]+", ".", s)

    s = trim(squeeze_blanks(s))
    sub("^[.]$", "", s)

    ## print "%% DEBUG: leave fix_note(): s = [" s "]"

    return (s)
}


function fix_pages(s,    t)
{
    ## Typical values:
    ##     "xxv, 612 p. :"
    ##     "xxv, 612 p. ;"
    ##     "128 S : $b Ill."
    ##     "1 online resource (xvi, 695 p.) : $b ill."
    ##     "1 online resource (xvi, 695 s.) $b ill."
    ##     "1 online resource (xxxiv, 329 p., [22] p. of plates) : $b ill."
    ##     "1 online resource."
    ##     "1 vol. (XVI-695 p.) : $b ill. ; $c 26 cm."
    ##     "1 vol. (xxxiv-329 p., [22] p. de pl.) : $b ill. ; $c 21 cm."
    ##     "128 S : $b Ill."
    ##     "128 p. : $b ill. ; $c 22 cm."
    ##     "2 v. : $b port. ; $c 24 cm."
    ##     "218 p ; $c 21 cm : $b ill."
    ##     "218 p."
    ##     "218 p. : $b graph. ; $c 21 cm."
    ##     "218 p. ; $c 21 cm."
    ##     "258 p. ; $c 19 cm."
    ##     "365 pages."
    ##     "6 v. : $b ill., portr. ; $c 25 cm."
    ##     "6 vols : $b Ill."
    ##     "63 p., [4] p. of plates : $b ill. (some col.) ; $c 20 cm."
    ##     "X, 604 S $b Ill $c 25 cm"
    ##     "X, 604 p. : $b ill. ; $c 25 cm."
    ##     "X-604 p. : $b photogr., couv. ill. en coul. ; $c 26 cm."
    ##     "XI, 317 p. : $b il.; $c 24 cm."
    ##     "XVI, 365 S. ; $c 8""
    ##     "XVI, 365 p. : $b il. ; $c 24 cm."
    ##     "XVI, 365 p. : $b ill. ; $c 24 cm."
    ##     "XVI, 365 str. : $b ilustr. ; $c 24 cm."
    ##     "XVI, 695 S $b Ill"
    ##     "XVI, 695 S. : $b ill."
    ##     "XVI, 695 str. : $b ilustr. ; $c 26 cm."
    ##     "XXXIV, 329 S $b Ill"
    ##     "XXXIV, 329 S : $b ill."
    ##     "XXXIV, 329 s., 22 s. fot. ; $c 21 cm."
    ##     "x, 604 p. : $b ill., ports ; $c 25 cm."
    ##     "x, 604 p. : $b ill., ports. ; $c 25 cm."
    ##     "x, 604 p. : $b ill., ports. ; $c 26 cm."
    ##     "x, 604 str. : $b ilustr. ; $c 26 cm."
    ##     "xi, 317 p., [11] leaves of plates : $b ill. ; $c 24 cm."
    ##     "xvi, 365 p, : $b ill. ; $c 24 cm."
    ##     "xvi, 365 p. $b ill. $c 24 cm."
    ##     "xvi, 365 p. : $b ill. ; $c 24 cm."
    ##     "xvi, 395 p. : $b ill. (plates)."
    ##     "xvi, 695 p. : $b ill. ; $c 25 cm."
    ##     "xvi, 695 p. : $b ill. ; $c 26 cm."
    ##     "xvi, 695 str. : $b ilustr. ; $c 26 cm."
    ##     "xxxiv, 329 p., [22] p. of plates : $b ill. ; $c 21 cm."
    ##     "xxxiv, 329 pp., ill."
    ##     "xxxiv + 329, 22fot."

    ## print "%% DEBUG: enter fix_pages(\"" s "\")"

    s = tolower(s)
    sub(" *[:;] *$",			"",	s)
    gsub(" [ps][.] *",			"",	s)
    sub(" [ps]$",			"",	s)

    if (match(s, "^ *[ivxlcdm]+ *, *[0-9]+"))
	sub(" *, *",			" + ", s)

    sub("in +various +pagings",		" (various) ",	s)
    sub("various +pagings",		" (various) ",	s)
    sub("p[.] *cm[.]",			"????",		s)
    sub("^ *[ps][.]? *$",		"????",		s)
    sub("^ *[0-9]+ *v[.]? *$",		"????",		s)	# "4 v." to "????"
    sub("^ *v[.]? *<[- 0-9]+> *$",	"????",		s)	# "v. < 1->" to "????"
    sub(";? *[0-9]+ *cm[.]?",		"",		s)	# discard book size

    ## print "%% DEBUG: 1: in fix_pages(\"" s "\")"

    s = (" " s " ")

    gsub("[(]", " ( ", s)
    gsub("[)]", " ) ", s)

    sub("[]] *",			"] ",		s)
    sub("1 online resource[.]? *",	" ",		s)
    sub("[(] *plates *[)]",		" ",		s)
    sub(" +pages[.]?",			" ",		s)
    sub(" +str[.] +",			" ",		s)
    sub(" +leaves of plates +",		" ",		s)
    sub(" +of +plates +",		" ",		s)
    sub(" +plates +",			" ",		s)
    sub(" +de +platos +",		" ",		s)
    sub(" +de +pl[.] +",		" ",		s)

    s = trim(squeeze_blanks(s))

    ## print "%% DEBUG: 2: in fix_pages(\"" s "\")"

    if (s == "v.")
	s = "????"

    ## carmin.sudoc.abes.fr often has
    ##     "1 vol. (XXII-534 p.)"
    ## which has now been reduced to
    ##     "1 vol. (xxii-534)"

    gsub("^1 vol[.]? *", "", s)

    if (match(s, "[ivxlcdm]+-[0-9]"))	# convert "xix-321" to to "xix + 321"
	s = (substr(s, 1, RSTART + RLENGTH - 3) " + " substr(s, RSTART + RLENGTH - 1))

    if (match(s, "^[(].*[)]$"))		# reduce "(xix + 321)" to "xix + 321"
	s = substr(s, 2, RLENGTH - 2)

    s = trim(squeeze_blanks(s))

    ## print "%% DEBUG: 3: in fix_pages(\"" s "\")"

    if (match(s, ", +\\[[0-9]+\\]$"))
    {
	## Reduce "63, [4]" to "63 + 4"

	t = substr(s, RSTART + 1)
	s = substr(s, 1, RSTART - 1)
	gsub("[][]", " ", t)
	s = (s " + " t)
	s = trim(squeeze_blanks(s))
    }

    gsub(" *[+] *", " + ", s)

    sub(" *fot[.]?", "", s)		# convert text like ""xxxiv + 329, 22fot." to "xxxiv + 329, 22"
    sub(" *pp[.], +ill[.] *$", "", s)	# convert text like "xxxiv + 329 pp., ill." to "xxxiv + 329"


    ## print "%% DEBUG: 4: in fix_pages(\"" s "\")"

    if (s ~ "^[ivxlcdm]+ *, *[0-9]+$")
	sub(" *, *", " + ", s)		# convert text like "x, 507" to "x + 507"

    if (s ~ "^[ivxlcdm]+ *[+,] *[0-9]+ *[+,] *[0-9]+$")
	gsub(" *, *", " + ", s)		# convert text like "x, 507, 23" to "x + 507 + 23"

    ## print "%% DEBUG: 5: in fix_pages(\"" s "\")"

    if (s ~ "^[ivxlcdm]+ *[+,] *[0-9]+ *[+,] *\\[[0-9]+\\]$")
    {
	gsub(" *, *", " + ", s)		# convert text like "x, 507, 23" to "x + 507 + 23"
	gsub("[][]", " ", s)		# convert text like "xxxiv + 329 + [22]" to "xxxiv + 329 + 22"
    }

    if (s ~ "^[0-9]+ +vols$")		# move volume count to note field
    {
	append_suffix("note", s)
	s = ""
    }

    ## print "%% DEBUG: leave fix_pages(): s = [" s "]"

    return (s)
}


function fix_publisher()
{
    if (("publisher" in Entry) && (Entry["publisher"] == "Project Gutenberg"))
    {
	if (Entry_Count == 1)
	{
	    print "%-----------------------------------------------------------------------"
	    print "% Publisher abbreviations:"
	    print ""
	    print "@String{pub-PROJECT-GUTENBERG   = \"Project Gutenberg\"}"
	    print "@String{pub-PROJECT-GUTENBERG:adr = \"P.O. Box 2782,"
	    print "                                     Champaign, IL 61825-2782,"
	    print "                                     USA\"}"
	    print ""
	}
	Entry["publisher"] = "pub-PROJECT-GUTENBERG"
	if (!("address" in Entry))
	    Entry["address"] = "pub-PROJECT-GUTENBERG:adr"
    }
}


function fix_quotes(s,    count,k)
{
    count = 0

    gsub("\\\\\"", "<\001DER_UMLAUT\001>", s)	# hide umlaut accents

    k = index(s, "\"")

    while (k > 0)
    {
	count++

	if ((count % 2) == 1)	# open quote
	    s = substr(s,1,k-1) "``" substr(s,k+1)
	else			# close quote
	    s = substr(s,1,k-1) "''" substr(s,k+1)

	k = index(s, "\"")
    }

    if ((count % 2) == 1)
	warning("Unbalanced double quotes in [" sample(s) "]")

    gsub("<\001DER_UMLAUT\001>", "\\\"", s)	# restore hidden umlaut accents

    return (s)
}


function fix_series()
{
    if (("series" in Entry) && (Entry["series"] == "Project Gutenberg"))
    {
	if (Entry_Count == 1)
	{
	    print "%-----------------------------------------------------------------------"
	    print "% Series abbreviations:"
	    print ""
	    print "@String{ser-PROJECT-GUTENBERG = \"Project Gutenberg\"}"
	    print ""
	    print "%-----------------------------------------------------------------------"
	    print "% Bibliography entries:"
	    print ""
	}
	Entry["series"] = "ser-PROJECT-GUTENBERG"
    }
    gsub("[ ]+[$][a-z][ ]+", " ", Entry["series"])	# remove MARC subfield markers
    gsub(" *; *", "; ", Entry["series"])
    gsub(" *: *", ": ", Entry["series"])
}


function fix_subject(s,    count,k,m,n,parts)
{
    ## Typical values:
    ## "TeX (Computer file); Computerized typesetting.; Mathematics printing $x Computer programs."
    ## "Einstein, Albert, $d 1879-1955 $v Drama. $2 idszbzna"
    ## "Ulam, Stanislaw M. $2 swd"
    ## "Ulam, Stanislaw M. $d (1909-1984) $v dziennik intymny. $2 jhpk"

    gsub(" +[$][a-z] +", "; ", s)
    gsub(" *[.];", ";", s)
    sub(" *[.] *$", "", s)
    gsub(" *, *;", ";", s)

    n = split(s, parts, Suffix["subject"])
    m = 0
    s = ""

    for (k = 1; k <= n; ++k)		# remove duplicates, but preserve original subject order
    {
	count[tolower(parts[k])]++

	if (count[tolower(parts[k])] == 1)
	    s = (s == "") ? parts[k] : (s Suffix["subject"] parts[k])
    }

    ## Strip subject classification codes defined in
    ## http://www.loc.gov/standards/sourcelist/classification.html

    gsub("[$]2 +[a-z]+ *", " ", s)
    gsub(" *,", ",", s)
    gsub(" *;", ";", s)

    return (s)
}


function fix_subject_d(s)
{
    ## Reduce
    ##	"Shelley, Percy Bysshe,|d1792-1822."
    ## to
    ##	"Shelley, Percy Bysshe"
    ## save the dates in a separate entry,
    ## and return the result

    return (subvalue(s, "d", "subject-dates"))
}


function fix_TeX(s)
{
    gsub("[$]","\\$",s)
    gsub("[&]","\\&",s)
    gsub("[%]","\\%",s)

    return (s)
}


function fix_title(s)
{
    ## Do some final checks and fixups on non-proceedings titles

    ## print "%% DEBUG 1: fix_title(): s = [" s "]"

    s = trim(squeeze_blanks(s))

    ## print "%% DEBUG 2: fix_title(): s = [" s "]"

    if (s ~ "[a-z][.]$")
	warning("fix_title(): possible bogus final period that does not terminate an abbreviation [..." substr(s, length(s) - 9, 10) "]")

    ## reduce main heading qualifiers:
    ##
    ##     "Candid science III $h [Elektronische Ressource]" ->
    ##     "Candid science III "
    ##
    ##     "Fractals $h [electronic resource] : $b form, chance, and dimension" ->
    ##     "Fractals: $b form, chance, and dimension"

    gsub(" *\\\\?[$]h *[[][^][]*[]] *", " ", s)

    ## print "%% DEBUG 3: fix_title(): s = [" s "]"

    ## brace any German titles that we can recognize from common words that are not
    ## used in most other large languages

    s = (" " s " ")

    if (tolower(s) ~ "( als | auf | aus | bei | bis | der | die | das | dein | deine | drei |" \
		      " ein | eine | einer | eins | es | fur | f{\\\"u}r | ganz | ihre | ihrer | im | ist |" \
		      " mein | meine | mit | ober | oder | ohne | selbst | seiner | sind | teil |" \
	              " uber | {\\\"u}ber | und | unter | vom | von | zu | zum | zur | zwei | zwischen )")
    {
	s = ("{" trim(squeeze_blanks(s)) "}")

	if ( ! ("language" in Entry) )
	    Entry["language"] = "German"
    }

    ## print "%% DEBUG 4: fix_title(): s = [" s "]"

    return (trim(squeeze_blanks(s)))
}


function fix_title_b(s, t)
{
    ## Reduce
    ## "The Early Short Fiction|bPart One /|cEdith Wharton"
    ## to
    ## "The Early Short Fiction. Part One /|cEdith Wharton"
    ## and
    ## "A Voyage to Abyssinia /|cby Father Jerome Lobo /|btranslated from the French ..."
    ## to
    ## "A Voyage to Abyssinia /|cby Father Jerome Lobo"
    ## and
    ## "A Vanished Arcadia, |bbeing some account of the Jesuits in Paraguay, 1607 to 1767..."
    ## to
    ## "A Vanished Arcadia, being some account of the Jesuits in Paraguay, 1607 to 1767..."
    ## save a note entry, and
    ## return the result

    if (match(s,"/[|$]btranslated[^|$]*"))
    {
	t = "T" substr(s,RSTART+4,RLENGTH-4)
	s = substr(s,1,RSTART-1) substr(s,RSTART+RLENGTH)
	append_suffix("note", t)
    }
    else if (match(s,"[|$]bbeing"))
	s = substr(s,1,RSTART-1) substr(s,RSTART+2)
    else
	gsub("[|$]b",". ",s)

    ## print "DEBUG 1: fix_title(): s = [" s "]" > "/dev/stderr"

    # Remove "\$h [electronic resource]"
    gsub(" *.[$]h *\\[electronic resource\\] *", "", s)
    gsub(" *.[$]h *\\[Texte imprime\\] *", "", s)
    gsub(" *.[$]h *\\[Ressource electronique\\] *", "", s)

    gsub(" *.[$]h *electronic resource *", "", s)
    gsub(" *.[$]h *Texte imprime *", "", s)
    gsub(" *.[$]h *Ressource electronique *", "", s)

    ## print "DEBUG 2: fix_title(): s = [" s "]" > "/dev/stderr"

    return (s)
}


function fix_title_c(s,code, t)
{
    ## Reduce
    ##	"245 10|aAnna Karenina /|cby Leo Tolstoy/Tolstoi"
    ##  "245 00 $a Klassische Shell-Programmierung $b [automatisieren Sie Ihe Unix/Linux-Tasks] $c Arnold Robbins ; Nelson H. Beebe"
    ## to
    ##	"245 10|aAnna Karenina"
    ##  "245 00 $a Klassische Shell-Programmierung $b [automatisieren Sie Ihe Unix/Linux-Tasks]"
    ## save the "by ..." as a separate entry,
    ## and return the result

    if (match(s," */?[|$]c *(by)? *"))
    {
	t = substr(s,RSTART+RLENGTH)
	s = substr(s,1,RSTART-1)
	append_suffix(("USMARC-" code "-c"), fix_title_b(t))
	Last_Key = "USMARC"	# NB: override value saved by append_suffix()
    }

    return (s)
}


function fix_title_l(s, t)
{
    ## Reduce
    ##	"The Augsburg Confession,|lLatin, German"
    ## to
    ##	"The Augsburg Confession"
    ## save the language value, and
    ## return the result

    if (match(s,"[|$]l[^|$]*"))
    {
	t = substr(s,RSTART+2,RLENGTH-2)
	s = substr(s,1,RSTART-1) substr(s,RSTART+RLENGTH)
	append_suffix("language", t)
    }

    return (s)
}


function fix_URL_base(s)
{

    return ((s ~ "^[a-z]+://") ? s : (URL_Base s))
}


function fix_URL_q(s, t)
{
    ## Reduce
    ##	"http://www.library.adelaide.edu.au/etext/pg/etext95/fh104s10.zip|qmidi"
    ## to
    ##	"http://www.library.adelaide.edu.au/etext/pg/etext95/fh104s10.zip"
    ## save the URL contents, and
    ## return the result

    if (match(s,"[|$]q[^|$]*"))
    {
	t = substr(s,RSTART+2,RLENGTH-2)
	s = substr(s,1,RSTART-1) substr(s,RSTART+RLENGTH)
	append_suffix("URL-contents", t)
    }

    return (s)
}


function fix_URL_z(s)
{
    ## Reduce
    ##	"http://www.library.adelaide.edu.au/etext/pg/etext95/pge0112.zip|qZIP|zVolume 1 of 28"
    ## to
    ##	"http://www.library.adelaide.edu.au/etext/pg/etext95/pge0112.zip|qZIP (Volume 1 of 28)"
    ## return the result

    if (match(s,"[|$]z[^|$]*"))
	s = substr(s,1,RSTART-1) " (" substr(s,RSTART+2,RLENGTH-2) ") " substr(s,RSTART+RLENGTH)

    return (trim(s))
}


function fix_URL(s, j,k,n,words)
{
    ## Remove a generic URL that serves no useful purpose
    gsub("ftp://uiarchive.cso.uiuc.edu/pub/etext/gutenberg/;? *", "", s)

    ## The URLs are often duplicated, so eliminate the extra ones
    n = split(s, words, "; *")
    s = words[1]

    for (k = 2; k <= n; ++k)
    {
	dups = 0

	for (j = 1; j < k; ++j)
	{
	    if (words[j] == words[k])
		break
	}

	if (j >= k)
	    s = s "; " words[k]
    }

    return (s)
}


function fix_volume(s)
{
    ## print "%% DEBUG: enter fix_volume(\"" s "\")"

    s = trim(s)
    sub("^no[.] *", "", s)
    sub("^v(ol)?[.]? *", "", s)
    sub("[.] *$", "", s)

    ## print "%% DEBUG: leave fix_volume(): s = [" s "]"

    return (s)
}


function fix_year(s)
{
    ## print "%% DEBUG: enter fix_year(\"" s "\")"

    s = trim(s)
    sub("[.]+$", "", s)

    while (match(s, "\\[[12][0-9][0-9][0-9]\\]"))
	s = (substr(s, 1, RSTART - 1) substr(s, RSTART + 1, RLENGTH - 2) substr(s, RSTART + RLENGTH))

    sub("cop[.] *", "", s)

    if (s ~ "^c[12][0-9][0-9][0-9]$")
	s = substr(s, 2,4)

    ## French catalog at carmin.sudoc.abes.fr often has data like
    ## "DL 1989" or "1989." or "cop. 2005." or "impr. 2005"

    gsub("[.; ]+$", "", s)
    sub("^cop[.]? +", "", s)
    sub("^DL +", "", s)
    sub("^impr[.]? +", "", s)

    ## print "%% DEBUG: leave fix_year(): s = [" s "]"

    return (s)
}


function hyphenate_isbn_13(    i, isbn_10, isbn_13, j, k, n, parts, s, t)
{
    ## Library records seem to omit hyphenation in ISBN-13 values.  We
    ## can supply it correctly if the corresponding ISBN-10 value is
    ## hyphenated.

    isbn_10 = Entry["ISBN"]
    isbn_13 = Entry["ISBN-13"]
    gsub("-", "", isbn_13)		# discard any current hyphenation

    ## print "%% DEBUG: hyphenate_isbn_13(): isbn_10 = [" isbn_10 "]"
    ## print "%% DEBUG: hyphenate_isbn_13(): isbn_13 = [" isbn_13 "]"

    s = isbn_10
    gsub("[(][a-z]+[)]", "", s)	# ignore parenthesized comments
    n = split(s, parts)

    for (k = 1; k <= n; ++k)
    {
	t = parts[k]
	gsub("-", "", t)

	## print "%% DEBUG: hyphenate_isbn_13(): parts[" k "] = [" parts[k] "]"
	## print "%% DEBUG: hyphenate_isbn_13(): t = [" t "]"

	if (t ~ "^[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9Xx]$") # ISBN-10 pattern
	{
	    i = index(isbn_13, ("978" substr(t,1,9)))

	    if (i > 0)			# matched leading 12 digits of hyphenless isbn-13
	    {
		isbn_13 = (substr(isbn_13, 1, i + 2) "-" substr(isbn_13, i + 3))
		i += 4			# advance past "978-"

		for (j = 1; j < 13; ++j)
		{
		    ## print "%% DEBUG: hyphenate_isbn_13(): j = [" j "]  rem10 = [" substr(parts[k], j) "]  rem13 = [" substr(isbn_13, i) "]"

		    if (substr(parts[k], j, 1) == "-")
			isbn_13 = (substr(isbn_13, 1, i - 1) "-" substr(isbn_13, i))

		    ++i
		}

		gsub("-x", "-X", isbn_13) # ensure uppercase form of ISBN-13 value
	    }
	}
    }

    Entry["ISBN-13"] = isbn_13
}

function initialize()
{
    ## NB: This MUST include a trailing slash
    URL_Base = "ftp://uiarchive.cso.uiuc.edu/pub/etext/gutenberg/"

    "date" | getline Current_Date_and_Time
    close("date")

    ## For the Project Gutenberg USMARC list, which is ordered by
    ## descending volume number, there are several labels which appear
    ## more than once.  This table records their frequencies for use in
    ## unique_citation_label() to generate a unique suffix letter.

    initialize_label_frequency()
    initialize_ignore_list()

    ## Multiple USMARC records of the same number are joined with these
    ## suffixes separating them:

    Suffix["abstract"] 		= ". "
    Suffix["author"] 		= " and "
    Suffix["author-dates"]	= "; "
    Suffix["editor"] 		= " and "
    Suffix["ISBN"] 		= ", "
    Suffix["ISBN-13"] 		= ", "
    Suffix["ISSN"] 		= ", "
    Suffix["ISSN-L"] 		= ", "
    Suffix["LCCN"] 		= "; "
    Suffix["MRclass"] 		= ", "
    Suffix["language"] 		= "; "
    Suffix["note"] 		= ". "
    Suffix["publisher"] 	= " "
    Suffix["series"] 		= "; "
    Suffix["subject"] 		= "; "
    Suffix["subject-dates"]	= "; "
    Suffix["title"] 		= " "
    Suffix["translator"] 	= "; "
    Suffix["URL"] 		= "; "
    Suffix["URL-contents"] 	= "; "
    Suffix["USMARC"] 		= "; "
    Suffix["USMARC-700"] 	= "; "
    Suffix["USMARC-740"] 	= "; "
    Suffix["USMARC-745"] 	= "; "
    Suffix["xxauthor"] 		= " and "
    Suffix["xxeditor"] 		= " and "
}


function initialize_ignore_list()
{
    ## List of words to ignore in forming citation tags.  The initial
    ## list was extracted from the bibindex badwords list, and covers
    ## a few European languages as well as English.
    Ignore["a"]         = 1
    Ignore["ab"]        = 1
    Ignore["aber"]      = 1
    Ignore["als"]       = 1
    Ignore["an"]        = 1
    Ignore["and"]       = 1
    Ignore["are"]       = 1
    Ignore["as"]        = 1
    Ignore["auf"]       = 1
    Ignore["aus"]       = 1
    Ignore["az"]        = 1
    Ignore["bei"]       = 1
    Ignore["bir"]       = 1
    Ignore["but"]       = 1
    Ignore["da"]        = 1
    Ignore["das"]       = 1
    Ignore["dat"]       = 1
    Ignore["de"]        = 1
    Ignore["dei"]       = 1
    Ignore["dem"]       = 1
    Ignore["den"]       = 1
    Ignore["der"]       = 1
    Ignore["des"]       = 1
    Ignore["det"]       = 1
    Ignore["di"]        = 1
    Ignore["die"]       = 1
    Ignore["dos"]       = 1
    Ignore["e"]         = 1
    Ignore["een"]       = 1
    Ignore["eene"]      = 1
    Ignore["egy"]       = 1
    Ignore["ei"]        = 1
    Ignore["ein"]       = 1
    Ignore["eine"]      = 1
    Ignore["einen"]     = 1
    Ignore["einer"]     = 1
    Ignore["eines"]     = 1
    Ignore["eit"]       = 1
    Ignore["el"]        = 1
    Ignore["en"]        = 1
    Ignore["er"]        = 1
    Ignore["es"]        = 1
    Ignore["et"]        = 1
    Ignore["ett"]       = 1
    Ignore["eyn"]       = 1
    Ignore["eyne"]      = 1
    Ignore["for"]       = 1
    Ignore["from"]      = 1
    Ignore["fuer"]      = 1
    Ignore["fur"]       = 1
    Ignore["gl"]        = 1
    Ignore["gli"]       = 1
    Ignore["ha"]        = 1
    Ignore["haben"]     = 1
    Ignore["had"]       = 1
    Ignore["hai"]       = 1
    Ignore["has"]       = 1
    Ignore["hat"]       = 1
    Ignore["have"]      = 1
    Ignore["he"]        = 1
    Ignore["heis"]      = 1
    Ignore["hen"]       = 1
    Ignore["hena"]      = 1
    Ignore["henas"]     = 1
    Ignore["het"]       = 1
    Ignore["hin"]       = 1
    Ignore["hinar"]     = 1
    Ignore["hinir"]     = 1
    Ignore["hinn"]      = 1
    Ignore["hith"]      = 1
    Ignore["ho"]        = 1
    Ignore["hoi"]       = 1
    Ignore["i"]         = 1
    Ignore["il"]        = 1
    Ignore["in"]        = 1
    Ignore["is"]        = 1
    Ignore["ist"]       = 1
    Ignore["ka"]        = 1
    Ignore["ke"]        = 1
    Ignore["l"]         = 1
    Ignore["la"]        = 1
    Ignore["las"]       = 1
    Ignore["le"]        = 1
    Ignore["les"]       = 1
    Ignore["lo"]        = 1
    Ignore["los"]       = 1
    Ignore["mia"]       = 1
    Ignore["mit"]       = 1
    Ignore["n"]         = 1
    Ignore["na"]        = 1
    Ignore["nji"]       = 1
    Ignore["not"]       = 1
    Ignore["o"]         = 1
    Ignore["oder"]      = 1
    Ignore["of"]        = 1
    Ignore["on"]        = 1
    Ignore["or"]        = 1
    Ignore["os"]        = 1
    Ignore["others"]    = 1
    Ignore["s"]         = 1
    Ignore["sie"]       = 1
    Ignore["sind"]      = 1
    Ignore["so"]        = 1
    Ignore["t"]         = 1
    Ignore["ta"]        = 1
    Ignore["the"]       = 1
    Ignore["to"]        = 1
    Ignore["um"]        = 1
    Ignore["uma"]       = 1
    Ignore["un"]        = 1
    Ignore["una"]       = 1
    Ignore["und"]       = 1
    Ignore["une"]       = 1
    Ignore["uno"]       = 1
    Ignore["unter"]     = 1
    Ignore["von"]       = 1
    Ignore["with"]      = 1
    Ignore["y"]         = 1
    Ignore["yr"]        = 1

#   Additional words added later
    Ignore["also"]      = 1
    Ignore["any"]       = 1
    Ignore["away"]      = 1
    Ignore["by"]        = 1
    Ignore["cum"]       = 1
    Ignore["dans"]      = 1
    Ignore["down"]      = 1
    Ignore["into"]      = 1
    Ignore["its"]       = 1
    Ignore["off"]       = 1
    Ignore["onto"]      = 1
    Ignore["out"]       = 1
    Ignore["over"]      = 1
    Ignore["sur"]       = 1
    Ignore["that"]      = 1
    Ignore["these"]     = 1
    Ignore["this"]      = 1
    Ignore["those"]     = 1
    Ignore["unto"]      = 1
    Ignore["up"]        = 1
    Ignore["via"]       = 1
    Ignore["without"]   = 1
    Ignore["zu"]        = 1
    Ignore["zum"]       = 1
    Ignore["zur"]       = 1
}


function initialize_label_frequency()
{
}


function ISBN_Digit(the_ISBN,n)
{

    return (ISSN_Digit(the_ISBN,n))	# same digit values in ISBN and ISSN data
}


function is_corporate_name(s)
{

    return (match(s,/Ltd/) || match(s,/Co[.]/) || match(s,/Company/) || \
	    match(s,/Corp/) || match(s,/Inc[.]?$/) || match(s,/Pty[.]?/) || \
	    match(s,/Cie[.]/) || match(s,/AG/))
}


function isletter(c)
{

    return (index("ABCDEFGHIJKLMNOPQRSTUVWXYZ",toupper(c)) > 0)
}


function is_proceedings()
{
    return ( ("entrytype" in Entry) && (Entry["entrytype"] == "Proceedings"))
}

function ISSN_Digit(the_ISSN,n)
{

    return (index("0123456789X", substr(the_ISSN, n, 1)) - 1)
}


function isupper(c)
{

    return (index("ABCDEFGHIJKLMNOPQRSTUVWXYZ",c) > 0)
}


function new_entry()
{
    clear_array(Entry)
    ## Author_Editor_Key = "author"
    ## Last_Key = ""
    ## Entry["address"] = ""
    ET_AL = ""
}


function print_entry(    issn,k,key)
{
    ## print "%% DEBUG 5: Entry[\"editor\"]  = [" Entry["editor"] "]"

    if (!("title" in Entry))
    {
	new_entry()
	return
    }

    ## print "%% DEBUG: print_entry(): title     = [" Entry["title"] "]"
    ## print "%% DEBUG: print_entry(): publisher = [" Entry["publisher"] "]"

    fix_author_editor_confusion()

    Entry_Count++
    k = 0

    for (key in Entry)		# count the number of saved key/value pairs
    {
	Entry[key] = fix_quotes(Entry[key])
	k++
    }

    set("entrytype","Book")	# NB: force fixed type "@Book{...}" for now: may be changed later

    if ("author" in Entry)
    {
	Entry["author"] = (fix_author_editor_names(Entry["author"]) ET_AL)
	gsub(" +and +others +and +others *$", " and others ", Entry["author"])
    }

    if ("xxauthor" in Entry)
    {
	Entry["xxauthor"] = (fix_author_editor_names(Entry["xxauthor"]) ET_AL)
	gsub(" +and +others +and +others *$", " and others ", Entry["xxauthor"])
    }

    if ("editor" in Entry)
    {
	Entry["editor"] = (fix_author_editor_names(Entry["editor"]) ET_AL)
	gsub(" +and +others +and +others *$", " and others ", Entry["editor"])
    }

    if ("xxeditor" in Entry)
    {
	Entry["xxeditor"] = (fix_author_editor_names(Entry["xxeditor"]) ET_AL)
	gsub(" +and +others +and +others *$", " and others ", Entry["xxeditor"])
    }

    check_for_proceedings()

    if (("entrytype" in Entry) && (k > 0))
    {
	## These two handle the special case of Project Gutenberg USMARC records
	## and must be called before any output is done
	fix_publisher()
	fix_series()

	clean_entry()

	if (entry_name() == "InProceedings")
	    print_Proceedings()	# want InProceedings to crossref a Proceedings

	print_line("@" entry_name() "{" citation_label() ",")

	if (entry_name() == "Phone")
	{
	    delete Entry["author"]
	    delete Entry["title"]
	}

	## These two must come AFTER the call to citation_label(), because
	## that function assumes the original "Last, First Middle" order
	## of names, while these functions change them to BibNet standard
	## order "First Middle Last".

	fix_author_order()
	fix_editor_order()

	## Discard any identical information from the MARC 245 field

	if ( ("author" in Entry) && ("USMARC-245-c" in Entry) && (Entry["author"] == Entry["USMARC-245-c"]) )
	    delete Entry["USMARC-245-c"]

	if ( ("editor" in Entry) && ("USMARC-245-c" in Entry) && (Entry["editor"] == Entry["USMARC-245-c"]) )
	    delete Entry["USMARC-245-c"]

	if (is_proceedings() && ("author" in Entry) && ((!("editor" in Entry)) || (Entry["editor"] == "")) )
	{
	    Entry["editor"] = Entry["author"]
	    delete Entry["author"]
	}

	if (("publisher" in Entry) && (Entry["publisher"] == "pub-PROJECT-GUTENBERG"))
	    Entry["acknowledgement"] = "ack-nhfb # \" and \" # ack-st"
	else
	    Entry["acknowledgement"] = "ack-nhfb"

	Entry["bibdate"] = Current_Date_and_Time

	## Change "not available" to something more recognizable

	if (("volume" in Entry) && (Entry["volume"] == "na"))
	    Entry["volume"] = "n/a"

	## Change note to remark.

	if ("note" in Entry)
	{
	    Entry["remark"] = Entry["note"]
	    delete Entry["note"]
	}

	if (is_proceedings())
	{
	    Entry["title"]     = "{" Entry["title"] "}"
	    Entry["booktitle"] = "{" Entry["booktitle"] "}"
	}
	else
	{
	    Entry["title"]     = fix_title(Entry["title"])
	    Entry["booktitle"] = fix_title(Entry["booktitle"])
        }

	if ("pages_whole" in Entry)
	    Entry["pages_whole"] = fix_pages(Entry["pages_whole"])

	if ("pages" in Entry)
	    Entry["pages"] = fix_pages(Entry["pages"])

	## After trying strict alphabetic order for the keywords, I
	## decided that it is more useful to have most of them in a
	## specific order, that corresponding to the usual order in an
	## article citation.  print_pair() deletes each entry that
	## it prints, and ignores any that don't exist, so that key/value
	## pairs are not duplicated in the subsequent output after
	## sort_keys().  The key order here is chosen to match biborder(1).

	print_pair("author")
	print_pair("editor")
	print_pair("key")
	print_pair("booktitle")
	print_pair("title")
	print_pair("crossref")
	print_pair("chapter")
	print_pair("journal")
	print_pair("volume")
	print_pair("type")
	print_pair("number")
	print_pair("howpublished")
	print_pair("institution")
	print_pair("organization")
	print_pair("publisher")
	print_pair("school")
	print_pair("address")
	print_pair("edition")
	print_pair("pages_whole")	# NB: Must precede "pages" because it
	print_pair("pages")		# checks for the existence of "pages"
	print_pair("day")
	print_pair("month")
	print_pair("year")
	print_pair("CODEN")
	print_pair("ISBN")
	print_pair("ISBN-13")

	issn = ("ISSN" in Entry) ? Entry["ISSN"] : ""

	print_pair("ISSN")

	if ("ISSN-L" in Entry)
	{
	    ## print "%% DEBUG : ISSN = [" issn "]  ISSN-L = [" Entry["ISSN-L"] "]"
	    ## For most MARC records, the ISSN-L value may be
	    ## identical to the ISSN value, so there seems no point in
	    ## recording a duplicate value it.  We output it only if
	    ## its value differs from the ISSN.

	    if (issn == "")
		print_pair("ISSN-L")
	    else if (issn != Entry["ISSN-L"])
		print_pair("ISSN-L")
	    else
		delete Entry["ISSN-L"]
	}

	print_pair("LCCN")
	print_pair("bibdate")
	print_pair("bibsource")
	print_pair("note")
	print_pair("price")
	print_pair("series")
	print_pair("acknowledgement")
	print_pair("subject")
	print_pair("ajournal")
	print_pair("fjournal")

	## Output any remaining key/value pairs sorted by key name
	sort_keys()

	for (k = 1; Sorted_Keys[k]; ++k)
	{
	    if (Entry[Sorted_Keys[k]] != "")
	    {
		if (Sorted_Keys[k] ~ "^USMARC")
		{
		    if (USMARC == "yes")
		    {
			gsub("[$]", "\\$", Entry[Sorted_Keys[k]])
			print_key_value(Sorted_Keys[k],Entry[Sorted_Keys[k]])
		    }
		}
		else
		    print_key_value(Sorted_Keys[k],Entry[Sorted_Keys[k]])
	    }
	}

	print_line("}\n")
    }

    new_entry()
}


function print_key_abbrev(key,abbrev)
{
    print_line(sprintf("  %s =%s %s,", key, \
	substr("                ",1,12-length(key)), abbrev))
}


function print_key_string(key,string)
{
    ## TeXify 8-bit accented characters
    gsub(/\252/,"\\?",string)	# what is this character?? Perhaps trademark
    gsub(/\260/,"\\?",string)	# what is this character?? Displays as Angstrom accent in willow, but as separate character
    gsub(/\341/,"\\`",string)
    gsub(/\342/,"\\'",string)
    gsub(/\343/,"\\^",string)
    gsub(/\346/,"\\?",string)	# what is this character??
    gsub(/\351/,"\\v ",string)
    gsub(/\365x/,"${}^x$",string)
    gsub(/\365X/,"${}^X$",string)
    gsub(/\350a/,"{\\\"a}",string)
    gsub(/\350e/,"{\\\"e}",string)
    gsub(/\350o/,"{\\\"o}",string)
    gsub(/\350u/,"{\\\"u}",string)

    gsub(/[.][.][.]/,"\\ldots{}",string) # convert ellipses ... to \ldots{}

    gsub(/[\;:, ]+$/,"",string)	# remove selected trailing punctuation

    print_line(sprintf("  %s = %s\"%s\",", \
	key, \
	substr("                ", 1, 12 - length(key)), \
	trim(fix_brace_level(enquote(string)))))
}


function print_key_value(key,value)
{
    if (key == "entrytype")		# already output as @type{tag, # }
	return
    else if (key == "acknowledgement")
	print_key_abbrev(key,value)
    else if ((key == "publisher") && ("publisher" in Entry) && (Entry["publisher"] ~ "pub-"))
	print_key_abbrev(key,value)
    else if ((key == "series") && ("series" in Entry) && (Entry["series"] ~ "ser-"))
	print_key_abbrev(key,value)
    else if ((key == "address") && ("address" in Entry) && (Entry["address"] ~ "pub-"))
	print_key_abbrev(key,value)
    else if (key == "month")
	print_month(value)
    else if (key == "pages_whole")
	print_key_string(("pages" in Entry) ? "pageswhole" : "pages", value)
    else
	print_key_string(key,value)
}


function print_line(line)
{
    print line
}


function print_month(month, mon)
{
    warning("print_month not yet implemented")

    return

    mon = tolower(month)
    gsub(/[^A-Za-z\055]/,"",mon)	# strip punctuation

    if (mon in month_abbrev)
	print_key_abbrev("month", month_abbrev[mon])
    else
	print_key_string("month",month)
}


function print_pair(key)
{
    if (key in Entry)
    {
	if (Entry[key] != "")
	    print_key_value(key,Entry[key])

	delete Entry[key]
    }
}


function print_Proceedings()
{
    warning("print_Proceedings() not yet implemented")
}


function sample(s)
{
    ## return up to 256 chars of s

    return ((length(s) > 256) ? (substr(s,1,256) "...") : s)
}


function set(key,value)
{
    Entry[key] = value
    Last_Key = key
}


function sort_keys( k,key,m,n)
{
    clear_array(Sorted_Keys)

    n = 0

    for (key in Entry)
    {
	n++
	Sorted_Keys[n] = key
    }

    for (k = 1; k < n; ++k)
    {
	for (m = k + 1; m <= n; ++m)
	{
	    if (tolower(Sorted_Keys[k]) > tolower(Sorted_Keys[m]))
	    {
		key = Sorted_Keys[m]
		Sorted_Keys[m] = Sorted_Keys[k]
		Sorted_Keys[k] = key
	    }
	}
    }
}


function squeeze_blanks(s)
{
    gsub(/[ \t][ \t]+/," ",s)

    return (s)
}


function subvalue(s,letter,key, t)
{
    if (match(s,"[|$]" letter "[^|$]*"))
    {
	t = substr(s,RSTART+2,RLENGTH-2)
	s = substr(s,1,RSTART-1) substr(s,RSTART+RLENGTH)

	if (key ~ "(-dates|volume)$") # change 1771-1832 to 1771--1832
	{
	    gsub("-+","--",t)
	    gsub("[.]+$","",t)
	}

	append(key, trim(t), "; ")
    }

    return (s)
}


function terminate()
{
    if (("author" in Entry) || ("title" in Entry))
	print_entry();
}


function trim(s)
{
    gsub(/^[ \t]+/,"",s)
    gsub(/[ \t]+$/,"",s)

    return (s)
}


function unique_citation_label(label)
{
    ## For the Project Gutenberg USMARC list, supply a suffix letter to
    ## make citation labels unique.

    if ((label in Label_Frequency) && \
	("publisher" in Entry) && \
	(Entry["publisher"] == "pub-PROJECT-GUTENBERG"))
    {
	if (label in Label_Count)
	    Label_Count[label]--
	else
	    Label_Count[label] = Label_Frequency[label]

	if (Label_Count[label] <= 0)
	    warning("Label_Frequency[\"" label "\"] is wrong in initialize()")
	else
	    label = label substr("abcdefghijklmnopqrstuvwxyz", Label_Count[label], 1)
    }

    ## Check for duplicate citation labels

    if (label in Label_Used)
	warning("duplicate citation label \"" label "\": add or increase Label_Frequency[] entry in initialize_label_frequency()")

    Label_Used[label]++

    return (label)
}


function unrecognized(code,s)
{
    if (s ~ "^[0-9][0-9][0-9] ")
    {
	## SUPPRESS FOR NOW: warning("unrecognized USMARC data: " s)
	s = trim(s)

	if (length(s) > 0)
	    append(("USMARC-" code), s, " ")
    }
}


function USMARC_020(s,    isbn13_pattern, k,n,parts,t)
{				# INTERNATIONAL STANDARD BOOK NUMBER (R)
    ## Typical values:
    ##
    ##	"020 $a 0070431523 (pbk.) : $c $19.95"
    ##	"020    $a 9781568812731 (alk. paper)"
    ##
    ##	"020    $a 0226823369 (cloth : alk. paper)"
    ##	"020    $a 0226823377 (pbk. : alk. paper)"
    ##	"020    $a 9780226823362 (cloth : alk. paper)"
    ##	"020    $a 9780226823379 (pbk. : alk. paper)"
    ##  "020    $a 28-417-7375-2 ( Edition imprimee)"
    ##	"020    $a 2-84177-375-2 (br)"

    ## print "%% DEBUG: enter USMARC_020(\"" s "\")"

    gsub(",? *: *alk[.] *paper", "", s)	# ignore "alkaline paper" comment

    gsub("cloth",	"hardcover", s)
    gsub("hbk[.]",	"hardcover", s)
    gsub("pbk[.]",	"paperback", s)

    isbn13_pattern = "[^0-9]978[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9Xx][^0-9]"

    ## print "%% DEBUG 1: USMARC_020(): s = [" s "]"

    if (match(s, "[$]c .*"))
    {				# cannot use subvalue here because of dollar sign in value
	t = substr(s, RSTART + 3)
	s = substr(s, 1, RSTART - 1)
	sub(" *: *$", "", s)
	gsub("[$]", "US\\$", t)
	append_suffix("price", t)
    }

    ## print "%% DEBUG 2: USMARC_020(): s = [" s "]"
    ## print "%% DEBUG 3: USMARC_020(): ISBN = [" Entry["ISBN"] "]"

    t = value(s)

    ## print "%% DEBUG: USMARC_020(): t = [" t "]"

    ## We should now have something like "0226823369 (cloth)" or "9780226823379 (paperback)"

    if (match((" " t " "), isbn13_pattern))
    {
	check_ISBN_13_list(t)
	append_suffix("ISBN-13", t)
    }
    else
    {
	check_ISBN_list(t)
	append_suffix("ISBN", t)
    }

    if ("ISBN" in Entry)
	Entry["ISBN"] = fix_ISBN(Entry["ISBN"])

    if ("ISBN-13" in Entry)
	Entry["ISBN-13"] = fix_ISBN(Entry["ISBN-13"])

    ## print "%% DEBUG 4: USMARC_020(): s = [" s "]"
    ## print "%% DEBUG 5: USMARC_020(): ISBN = [" Entry["ISBN"] "]"
    ## print "%% DEBUG 6: USMARC_020(): ISBN-13 = [" Entry["ISBN-13"] "]"
    ## print "%% DEBUG: leave USMARC_020() subvalue = [" subvalue(s, "c", "price") "]"
}


function USMARC_021(s)
{
    USMARC_020(s)
}


function USMARC_022(s,    t)
{				# INTERNATIONAL STANDARD SERIAL NUMBER (R)
    ## Format: 020	1073-922X $y 0749-2839
    ##         022 0  $a 0196-5204 $l 0196-5204 $2 1
    ## $y means incorrect ISSN
    ## $z means canceled ISSN
    ##
    ##    From http://www.loc.gov/marc/bibliographic/bd022.html:
    ##
    ##	    022 - International Standard Serial Number (R)
    ##
    ##    MARC 21 Bibliographic - Full                            October 2007
    ##    First Indicator                                         Second Indicator
    ##    Level of international interest                         Undefined
    ##    # - No level specified                                  # - Undefined
    ##    0 - Continuing resource of international interest
    ##    1 - Continuing resource not of international interest
    ##
    ##    Subfield Codes
    ##    $a - International Standard Serial Number (NR)          $z - Canceled ISSN (R)
    ##    $l - ISSN-L (NR)                                        $2 - Source (NR)
    ##    $m - Canceled ISSN-L (R)                                $6 - Linkage (NR)
    ##    $y - Incorrect ISSN (R)                                 $8 - Field link and sequence number (R)

    if (match(s, "[$]l +[0-9][0-9][0-9][0-9]-[0-9][0-9][0-9][0-9Xx]"))
    {
	t = trim(substr(s, RSTART + 2, RLENGTH - 2))
	check_ISSN_list(t)
	append_suffix("ISSN-L", t)
    }

    gsub("[$][0-9] *[-0-9]+", "", s)
    gsub("[$][lyz] +[0-9][0-9][0-9][0-9]-[0-9][0-9][0-9][0-9Xx]", "", s)
    s = value(s)
    check_ISSN_list(s)
    append_suffix("ISSN", s)
}


function USMARC_030(s)
{				# CODEN (Chemical Abstracts Serial Number)
    # Format: 030 	 $a JSOMAR $z JOSMAR
    # Key $a means valid CODEN, $z means canceled or invalid CODEN
    gsub("[$]z +[A-Z][A-Z][A-Z][A-Z][A-Z][A-Z0-9]", "", s)
    s = value(s)
    check_CODEN_list(s)
    append_suffix("CODEN", s)
}


function USMARC_050(s,    lccn)
{				# LIBRARY OF CONGRESS CALL NUMBER (R)
    lccn = value(s)

    ## print "%% DEBUG: USMARC_050(): s = [" s "]  lccn = [" lccn "]" > "/tmp/foo.dbg.001"

    if ( (lccn == "Internet") || (lccn == "") )
	;
    else if (!("LCCN" in Entry))
	append_suffix("LCCN", lccn)
    else if (index(Entry["LCCN"], lccn) == 0)
	append_suffix("LCCN", lccn)
}


function USMARC_084(s)
{				# Subject classification
    ## Typical values for MathSciNet codes
    ##     084    $a 01A60. $2 msc

    if (match(s, "[$]2 *msc"))
    {
        s = value(s)
	sub("[.]? *[$]2 *msc *$", "", s)
	append_suffix("MRclass", s)
    }
}


function USMARC_090(s)
{				# LIBRARY OF CONGRESS CALL NUMBER (R) [in University of California Melvyl system]
    USMARC_050(s)
}


function USMARC_097(s)
{				# LIBRARY OF CONGRESS CALL NUMBER (R) (not documented)
    USMARC_050(s)
}


function USMARC_100(s,    name)
{				# MAIN ENTRY--PERSONAL NAME (NR)
    ## print "%% DEBUG: enter USMARC_100(): Entry[" Last_Key "] = [" Entry[Last_Key] "] s = [" s "]"

    ## Typical values:
    ##     "100 1  $a Ulam, Stanislaw Marcin $d (1909-1984). $e Auteur."
    ##     "100 1  $a Ulam, Stanislaw M. $d (1909-1984)."
    ##     "100 1  $a Ulam, Stanislaw M. $d (1909 - 1984)."

    ## print "%% DEBUG 1: USMARC_100(): s = [" s "]"

    if (match(s, " [$]d [12][0-9][0-9][0-9] *- *[12][0-9][0-9][0-9] "))
    {
	years = substr(s, RSTART + 4, RLENGTH - 5)
	gsub(" ", "", years)
	gsub("-+", "--", years)

	## print "%% DEBUG 2: USMARC_100(): years = [" years "]"

	if (match(s, "[$]a[^$]*[$]"))
	    name = fix_name_order(substr(s, RSTART + 2, RLENGTH - 3))
	else
	    name = ""

	## print "%% DEBUG 3: USMARC_100(): name = [" name "]"

	append_suffix("author-dates", (name " (" years ")"))

	## print "%% DEBUG 4: USMARC_100(): author-dates = [" Entry["author-dates"] "]"
    }

    gsub("[.,]? *[$]e *Auteur[.]? *", " ", s)		# discard foreign $e subfield
    gsub("[.,]? *[$]e *Hrsg[.]? *", " ", s)		# discard foreign $e subfield

    if (match(s,"[|$]e[Ed][Dd][.] *$"))
    {
	s = substr(s,1,RSTART-1)
	Last_Key = "editor"
    }
    else
	Last_Key = "author"

    append_suffix(Last_Key, fix_author_editor_names(value(fix_author_d(fix_author_q(fix_author_c(s))))))

    ## print "%% DEBUG: leave USMARC_100(): Entry[" Last_Key "] = [" Entry[Last_Key] "] s = [" s "]"
}


function USMARC_110(s)
{				# MAIN ENTRY--CORPORATE NAME (NR)
    append_suffix("author", "{" value(s) "}")
}


function USMARC_111(s)
{				# MAIN ENTRY--MEETING NAME (NR)
    s = value(s)
    gsub(" +[$][a-z] +", " ", s)
    append_suffix("meetingname", s)
}


function USMARC_130(s)
{				# MAIN ENTRY--UNIFORM TITLE (NR)
    append_suffix("title", fix_title_l(fix_title_b(fix_title_c(value(s),"130"))))
}


function USMARC_210(s)
{				# Abbreviated Title
    s = (s " $")		# add trailing sentinel

    if (match(s, "[$]a *[^$]*[$]"))
	Entry["ajournal"] = substr(s, RSTART + 2, RLENGTH - 3)
}


function USMARC_222(s)
{				# Uniform Title (NR)
    s = (s " $")		# add trailing sentinel

    if (match(s, "[$]a *[^$]*[$]"))
	Entry["fjournal"] = substr(s, RSTART + 2, RLENGTH - 3)
}


function USMARC_245(s,    authors, editors, the_field, v)
{				# TITLE STATEMENT (NR)
    ## Typical values (line wrapped for readability)
    ##
    ##	"245 10 $a Pi-unleashed /
    ##          $c Jeorg Arndt, Christoph Haenel ; translated from the
    ##             German by Catriona and David Lischka."
    ##
    ##	"245 00 $a Designing data-intensive Web applications /
    ##	        $c Stefano Ceri ... [et al.]."
    ##
    ##	"245 00 $a Selected areas in cryptography :
    ##	        $b 12th international workshop, SAC 2005, Kingston, on,
    ##	           Canada, August 11-12, 2005, revised selected papers /
    ##	        $c [edited by] Bart Preneel, Stafford Tavares."
    ##
    ##  "245 00 $n 4A $a Combinatorial algorithms, part I"
    ##
    ##  "245 14 $a The C programming language /
    ##          $c Brian W. Kernighan, Dennis M. Ritchie."
    ##
    ##  "245 04 $a The Oxford-Duden German dictionary :
    ##          $b German-English, English-German /
    ##          $c edited by the Dudenredaktion and the German Section
    ##             of the Oxford University Press Dictionary Department ;
    ##             chief editors, W. Scholze-Stubenrecht, J.B. Sykes."

    the_field = "USMARC-245-c"

    if (match(s,"[^A-Za-z0-9]et  *al[^A-Za-z0-9]|[^A-Za-z0-9]et  *al$"))
	ET_AL = " and others"

    if (match(s, "translated from.*$"))
    {
	append_suffix("note", ("T" substr(s,RSTART+1)))
	s = delete_match(s)
    }

    if (match(s, "\\[?[Ee]dited by\\]?.*$"))
    {
	editors = substr(s, RSTART)
	s = delete_match(s)
	append_suffix(the_field, editors)
    }
    else if (match(s, "Editors?.*$"))
    {
	editors = substr(s, RSTART)
	s = delete_match(s)
	append_suffix(the_field, editors)
    }
    else if (match(s, "[$]c +.*$"))
    {
	## The $c subfield is required to be last, and usually
	## contains an ordered author list.  However, the author list
	## is normally provided by the MARC 100 and 700 fields, so we
	## store this list separately.

	authors = trim(substr(s, RSTART + 2))
	s = delete_match(s)
	gsub(" *, *", " and ", authors)

	## print "%% DEBUG 1: USMARC_245(): authors = [" authors "]"

	authors = fix_author_editor_names(authors)

	## print "%% DEBUG 2: USMARC_245(): authors = [" authors "]"

	if (the_field in Entry)
	    append_suffix(the_field, authors)
	else
	    set(the_field, authors)
    }

    if (the_field in Entry)
    {
	gsub("\\[ *et al.? *\\][.]?", " and others ", Entry[the_field])

	## print "%% DEBUG 1: USMARC_245(): Editor = [" Entry[the_field] "]"

	gsub(" +and +others +and +others +", " and others ", Entry[the_field])

	## print "%% DEBUG 2: USMARC_245(): Editor = [" Entry[the_field] "]"
    }

    if (match(s, "[$]n +[0-9]+[A-Z]* *"))
    {
	# "$n 4A " -> volume

	## print "%% DEBUG 1: USMARC_245(): s = [" s "]"

	v = substr(s, RSTART + 2, RLENGTH - 2)
	s = delete_match(s)
	v = trim(v)

	if ("volume" in Entry)
	    v = (Entry["volume"] "; " v)

	Entry["volume"] = trim(squeeze_blanks(v))

	## print "%% DEBUG 2: USMARC_245(): s = [" s "]"
    }

    append_suffix("title", fix_TeX(fix_title_b(fix_title_c(value(s),"245"))))

    ## print "%% DEBUG 4: Entry[" the_field "]  = [" Entry[the_field] "]"
}


function USMARC_250(s)
{				# EDITION STATEMENT (NR)
    ## Typical value:
    ##	"250 $a 4th ed."

    s = value(s)

    print "%% DEBUG: USMARC_250(): s = [" s "]"

    sub(" *[Ee]d[.]? *$", "", s)
    sub(" *[Ee]dition[.]? *$", "", s)
    sub(" *[Rr]ev[.]? *$", " revised", s)

    if (s ~ "^1st")
	return
    else if (s == "1.")
	return
    else if (s ~ "1[.] Aufl[.]")
	return
    else if (s == "2nd")
	s = "Second"
    else if (s == "3rd")
	s = "Third"
    else if (s == "4th")
	s = "Fourth"
    else if (s == "5th")
	s = "Fifth"
    else if (s == "6th")
	s = "Sixth"
    else if (s == "7th")
	s = "Seventh"
    else if (s == "8th")
	s = "Eighth"
    else if (s == "9th")
	s = "Nineth"
    else if (s == "10th")
	s = "Tenth"
    append_suffix("edition", s)
}


function USMARC_260(s)
{				# PUBLICATION, DISTRIBUTION, ETC. (IMPRINT) (NR)
    ## append_suffix("publisher", value(subvalue(s,"c","year")))
    subvalue(s, "c", "year")
    subvalue(s, "b", "publisher")
    subvalue(s, "a", "address")

    if (year in Entry)
	Entry["year"] = fix_year(Entry["year"])
}


function USMARC_300(s)
{				# PHYSICAL DESCRIPTION (R)
    ## $a - Extent (R): The number of pages, volumes, cassettes, total playing time, etc., of the described item.
    sub("[$][b-z].*$", "", s)
    append_suffix("pages", value(s))
}


function USMARC_440(s)
{				# SERIES STATEMENT/ADDED ENTRY--TITLE (R)
    append_suffix("series", value(subvalue(s,"v","volume")))
}


function USMARC_490(s)
{				# SERIES STATEMENT/ADDED ENTRY--TITLE (R)
    ## Typical value: "490 1 $a Computers and typesetting ; $v A",
    append_suffix("series", value(subvalue(s,"v","volume")))

}


function USMARC_500(s)
{				# GENERAL NOTE (R)
    s = value(s)

    if (substr(s,length(s),1) != ".")
	s = s "."

    append_suffix("note", s)
}


function USMARC_505(s)
{				# FORMATTED CONTENTS NOTE (NR)
    ## Typical value (from Classic Shell Scripting book)
    ## "505 00 $g 3
    ##     $t Searching and Substitutions $g 30 -- $g 4
    ##     $t Text Processing Tools $g 67 -- $g 5
    ##     $t Pipelines Can Do Amazing Things $g 87 -- $g 6
    ##     $t Variables, Making Decisions, and Repeating Actions $g 109 -- $g 7
    ##     $t Input and Output, Files, and Command Evaluation $g 140 -- $g 8
    ##     $t Production Scripts $g 177 -- $g 9
    ##     $t Enough awk to Be Dangerous $g 223 -- $g 10
    ##     $t Working with Files $g 267 -- $g 11
    ##     $t Extend Example: Merging User Databases $g 308 -- $g 12
    ##     $t Spellchecking $g 325 -- $g 13
    ##     $t Processes $g 352 -- $g 14
    ##     $t Shell Portability Issues and Extensions $g 381 -- $g 15
    ##     $t Secure Shell Scripts: Getting Started $g 413 -- $g A
    ##     $t Writing Manual Pages $g 423 -- $g B
    ##     $t Files and Filesystems $g 437 -- $g C
    ##     $t Important Unix Commands $g 473."

    append_suffix("tableofcontents", value(s) ".")

    gsub(" +[$]g +",	" ",		Entry["tableofcontents"])
    gsub(" +[$]g +",	" ",		Entry["tableofcontents"])
    gsub(" +[$]t +",	" : ",		Entry["tableofcontents"])
    gsub(" -- ",	" \\\\\n",	Entry["tableofcontents"])
    sub("[.]+$",	"",		Entry["tableofcontents"])
}


function USMARC_520(s)
{				# SUMMARY, ETC. (R)
    append_suffix("abstract", value(s) ".")
}


function USMARC_600(s)
{				# SUBJECT ADDED ENTRY--PERSONAL NAME (R)
    append_suffix("subject", fix_subject_d(value(s)))
}


function USMARC_630(s)
{				# SUBJECT ADDED ENTRY--UNIFORM TITLE (R)
    USMARC_600(s)
}


function USMARC_650(s)
{				# SUBJECT ADDED ENTRY--TOPICAL TERM (R)
    USMARC_600(s)
}


function USMARC_651(s)
{				# SUBJECT ADDED ENTRY--GEOGRAPHIC NAME (R)
    USMARC_600(s)
}


function USMARC_690(s)
{				# Extension: appears to contain list of subject areas
    ## Typical values:
    ##	"690    $a Computer science $v Textbooks"
    ##	"690    $a Mathematics and statistics $v Textbooks"

    USMARC_600(s)
}


function USMARC_700(s,    name, years)
{				# ADDED ENTRY--PERSONAL NAME (R)
				# used for authors after the first (set in USMARC_100())
				# but the MARC 100 field is sometimes absent
    ## Typical values:
    ##	"700 1  $a Ritchie, Dennis M., $d 1941-2011 $e joint author"
    ##	"700 1  $a Sykes, J. B. $q (John Bradbury)"

    ## print "%% DEBUG 1: enter USMARC_700(): Entry[" Last_Key "] = [" Entry[Last_Key] "] s = [" s "]"

    if (match(s, " [$]d [12][0-9][0-9][0-9] *- *[12][0-9][0-9][0-9] "))
    {
	years = substr(s, RSTART + 4, RLENGTH - 5)
	gsub(" ", "", years)
	gsub("-+", "--", years)

	if (match(s, "[$]a[^$]*[$]"))
	    name = fix_name_order(substr(s, RSTART + 2, RLENGTH - 3))
	else
	    name = ""

	append_suffix("author-dates", (name " (" years ")"))

	## print "%% DEBUG 2: USMARC_700(): author-dates = [" Entry["author-dates"] "]"
    }

    ## print "%% DEBUG 11: USMARC_700(): s = [" s "]"

    gsub("[$]e *joint *author *", " ", s)
    gsub("[.,]? *[$]e *Auteur[.]? *", " ", s)		# discard Polish $e subfield
    gsub("[.,]? *[$]e *Il[.] *", " ", s)		# discard Polish $e subfield (illustrator?)
    gsub("[.,]? *[$]e *Oprac[.] *", " ", s)		# discard Polish $e subfield (what is it?)
    gsub("[.,]? *[$]e *Wyb...o.r *", " ", s)		# discard Polish $e subfield (author?)
    gsub("[$]q *", " ", s)

    ## print "%% DEBUG 12: USMARC_700(): s = [" s "]"

    ## print "%% DEBUG 3: in    USMARC_700(): Entry[" Last_Key "] = [" Entry[Last_Key] "] s = [" s "]"

    ## "700 1  $a Mathews, William G. $e Wst\361ep." 		<-- Wst{\k{e}p (Polish) == Preface
    ## "700 1  $a Mycielski, Jan $d (1932- ). $e Oprac."	<-- Oprac. (Polish) == ????
    ## "700 1  $a Ulam, Fran\360coise. $e Pos\261."		<-- Postscriptum. (Polish) == Postscript

    gsub("[.,]? *[$]e *Wst{\\\\k{e}}p[.] *",			" [[Preface]] ", s)	# translate foreign $e subfield
    gsub("[.,]? *[$]e *Oprac[.] *",				" [[Oprac.]] ", s)	# translate foreign $e subfield
    gsub("[.,]? *[$]e *[Tt]rad[.] *",				" [[Translator]] ", s)	# translate foreign $e subfield
    gsub("[.,]? *[$]e *Pr{\\\\'e}face *",			" [[Preface]] ", s)	# translate foreign $e subfield
    gsub("[.,]? *[$]e *{\\\\'E}diteur scientifique[.]?*",	" [[Editor]] ", s)	# translate foreign $e subfield

    ## print "%% DEBUG 13: USMARC_700(): s = [" s "]"

    gsub("[.,]? *[$]e *Foreword *",				" [[Foreword]] ", s)	# show $e subfield
    gsub("[.,]? *[$]e *Introduction *",				" [[Introduction]] ", s)# show $e subfield
    gsub("[.,]? *[$]e *Pos[.] *$",				" [[Postscript]] ", s)	# show $e subfield
    gsub("[.,]? *[$]e *Preface *",				" [[Preface]] ", s)	# show $e subfield

    ## print "%% DEBUG 14: USMARC_700(): s = [" s "]"

    s = trim(s)

    ## print "%% DEBUG 15: USMARC_700(): s = [" s "]"

    ## print "%% DEBUG 4: in    USMARC_700(): Entry[" Last_Key "] = [" Entry[Last_Key] "] s = [" s "]"

    if (match(s,"[|$]e[Tt][Rr][.] *$"))
	append_suffix("translator", fix_author_editor_names(value(substr(s,1,RSTART-1))))
    else if (match(s,"[|$]eEd[.] *$"))
	append_suffix("editor", fix_author_editor_names(value(substr(s,1,RSTART-1))))
    else if ("author" in Entry)
	append_suffix("author", fix_author_editor_names(value(s)))
    else if ("editor" in Entry)
	append_suffix("editor", fix_author_editor_names(value(s)))
    else if (is_proceedings())
	append_suffix("editor", fix_author_editor_names(value(s)))	# no MARC 100 field found
    else
	append_suffix("author", fix_author_editor_names(value(s)))	# no MARC 100 field found

    ## print "%% DEBUG 5: leave USMARC_700(): Entry[" Last_Key "] = [" Entry[Last_Key] "] s = [" s "]"
}


function USMARC_740(s)
{				# 740 - ADDED ENTRY--UNCONTROLLED RELATED/ANALYTICAL TITLE (R)
    if ("title" in Entry)
    {
	sub("[.]", "", Entry["title"])
	Entry["title"] = Entry["title"] ": " value(s)
    }
    else
	append_suffix("USMARC-740", value(s))
}


function USMARC_745(s)
{				# 740 - ADDED ENTRY--UNCONTROLLED RELATED/ANALYTICAL TITLE (R)
    append_suffix("USMARC-745", value(s))
}


function USMARC_776(s,    lccn,k,n,parts)
{
    n = split(s, parts, " ")

    for (k = 1; k < n; ++k)
    {
	if (parts[k] == "$z")
	    append_suffix("ISBN", parts[k+1])
    }
}


function USMARC_852(s,    lccn,k,n,parts)	# http://www.loc.gov/marc/bibliographic/ecbdhold.html#mrcb852
{				# LOCATION (R)
    ## Typical values:
    ##	"852    $a SDB $b ek $h QA276.4 $i .K73 2005"

    ## Because this field provides supplemental location information,
    ## it is ignored if we already have an LCCN value (usually from a
    ## 050 record).  If we do not, then we require an indicator field
    ## of 0 (Library of Congress classification) in the 852 record before
    ## accepting it.  For example:
    ##
    ## Reject: 852    $a UkOxU $b Bodleian $b BOD Lower RR $h Phil.Fa.162K $7 18341056 $p N13267982 ...
    ##

    if (!("LCCN" in Entry))
    {
	sub("[$][b-z0-9].*$", "", s)
	lccn = ""
	n = split(s, parts, " ")

	for (k = 1; k < n; ++k)
	{
	    if (parts[k] == "$h")
		lccn = parts[k+1]
	    else if (parts[k] == "$i")
	    {
		if (((k + 2) <= n) && (parts[k+2] ~ "^[12][0-9][0-9][0-9]"))
		    lccn = (lccn " " parts[k+1] " " parts[k+2])
		else
		    lccn = (lccn " " parts[k+1])
	    }
	}

	## print "%% DEBUG: USMARC_852(): s = [" s "]  lccn = [" lccn "]" > "/tmp/foo.dbg.001"

	if ( (lccn == "Internet") || (lccn == "") )
	    ;
	else if (!("LCCN" in Entry))
	    append_suffix("LCCN", lccn)
	else if (index(Entry["LCCN"], lccn) == 0)
	    append_suffix("LCCN", lccn)
    }
}


function USMARC_856(s)		# http://www.loc.gov/marc/bibliographic/ecbdhold.html#mrcb856
{				# ELECTRONIC LOCATION AND ACCESS (R)
    ## Typical value:
    ##     859 7  $z Online version: $u http://epubs.siam.org/sam-bin/dbq/toclist/SIMA $2 http
    s = (s " $")		# add trailing sentinel

    if (s ~ "http://firstsearch.oclc.org/WebZ/")
	return			# ignore ugly WorldCat search URLs
    else if (s ~ "http://search.ebscohost.com/login.aspx")
	return			# ignore ugly EBSCO search URLs

    if (match(s, "[$]u *http://[^$ ]*"))
    {
	s = substr(s, RSTART + 2, RLENGTH - 2)
	gsub("&", "%26", s) # protect against bogus entity expansion

	if (!("URL" in Entry))
	    append_suffix("URL", s)
	else if (index(Entry["URL"], s) == 0)
	    append_suffix("URL", s)
    }

    ## OLD:    ## print "%% DEBUG: enter USMARC_856(\"" s "\")"
    ## OLD:    sub(/^ *[0-9]+ +[0-9]* +/,"",s)
    ## OLD:    ## print "%% DEBUG: clean USMARC_856(): s = [" s "]"
    ## OLD:    s = fix_URL_base(fix_URL_z(fix_URL_q(s)))
    ## OLD:    ## print "%% DEBUG: clean USMARC_856(): s = [" s "]"
    ## OLD:    gsub(" *[$][a-z0-9] +", " ; ", s)
    ## OLD:    ## print "%% DEBUG: clean USMARC_856(): s = [" s "]"
    ## OLD:    gsub("[A-Za-z]+[ ]", " ", s)	# remove words of prose
    ## OLD:    ## print "%% DEBUG: clean USMARC_856(): s = [" s "]"
    ## OLD:    gsub(" +; +", "; ", s)
    ## OLD:    gsub("; +;+", "; ", s)
    ## OLD:    append_suffix("URL", s)
    ## print "%% DEBUG: leave USMARC_856(): s = [" s "]"
}


function USMARC_859(s)
{
    USMARC_856(s)	# treat 856 and 859 identically until documentation of 859 is found
}


function value(s)
{
    ## Reduce
    ##	"100 1 |aGorky, Maxim"
    ## to
    ##	"Gorky, Maxim"
    ## by discarding the USMARC code, subcode, field letter, and trailing
    ## space, and return the result.

    sub(/^ *[0-9]+ +[0-9]* *[|$][a-z]/,"",s)

    return (trim(s))
}


function warning(message)
{
    print FILENAME ":" FNR ":%%" message >"/dev/stderr"
}
